#############################################################################
# Copyright (c) 2007-2013 Balabit
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 2 as published
# by the Free Software Foundation, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#
# As an additional exemption you are allowed to compile & link against the
# OpenSSL libraries as published by the OpenSSL project. See the file
# COPYING for details.
#
#############################################################################

from globals import *
from log import *
from control import *
from messagegen import syslog_prefix

def file_reader(fname):
    try:
        return open(fname + ".log", "r")
    except IOError as e:
        print_user("Error opening file: %s, %s" % (fname, str(e)))

def sql_reader(name):
    (db, table) = name
    try:
        return os.popen("""echo "select * from %s order by msg;" | sqlite3 -separator " "  %s """ % (table, db), "r")
    except OSError as e:
        print_user("Error opening file: %s, %s" % (fname, str(e)))


def check_contents(f, messages, syslog_prefix, skip_prefix):

    if not f:
        print_user("Failed to open file/database")
        return False
    print_user("Starting output file content check")

    lineno = 1
    # we handle message reordering by going through the complete
    # file and looking for the message
    # end the collected ranges must span the number of messages that
    # we expect
    matches = {}

    read_line = f.readline().strip()
    while read_line:
        prefix_len = len(syslog_prefix);

        if read_line[skip_prefix:prefix_len+skip_prefix] != syslog_prefix:
            print_user("message does not start with syslog_prefix=%s, file=%s:%d, line=%s" % (syslog_prefix, f, lineno, read_line))
            return False
        msg = read_line[skip_prefix+prefix_len:]
        m = re.match("^ (\S+) (\d+)/(\d+)", msg)

        if not m:
            print_user("message payload unexpected format, file=%s:%d, line=%s" % (f, lineno, read_line))
            return False
        msg = m.group(1)
        session = int(m.group(2))
        id = int(m.group(3))
        if (msg, session) not in matches:
            if id != 1:
                print_user("the id of the first message in a session is not 1, session=%d, id=%d, file=%s:%d, line=%s"  % (session, id, f, lineno, read_line))
                return False
        else:
            if matches[(msg, session)] != id - 1:
                print_user("message reordering/drop detected in the same session, session=%d, id=%d, expected_id=%d, file=%s:%d, line=%s" % (session, id, matches[(msg, session)]+1, f, lineno, read_line))
                return False
        matches[(msg, session)] = id
        read_line = f.readline().strip()
        lineno = lineno + 1

    for (msg, session, count) in messages:

        if (msg, session) not in matches:
            print_user("output log files lack this kind of message: %s session: %d, count: %d" % (msg, session, count))
            return False
        if matches[(msg, session)] != count - 1:
            print_user("not enough messages found, message: %s, session: %d, last_id: %d, count: %d" % (msg, session,  matches[(msg, session)], count))
            return False
        del matches[(msg, session)]

    if len(matches) > 0:
        print_user("output contains more messages than expected: %s" % str(matches))
        return False

    return True

def check_reader_expected(reader, messages, settle_time, syslog_prefix, skip_prefix):
    return check_contents(reader, messages, syslog_prefix, skip_prefix)

def check_file_expected(fname, messages, settle_time=1, syslog_prefix=syslog_prefix, skip_prefix = 0):
    print_user("Checking contents of output files: %s" % fname)
    flush_files(settle_time)
    return check_reader_expected(file_reader(fname), messages, settle_time, syslog_prefix, skip_prefix)

def check_sql_expected(dbname, tablename, messages, settle_time=1, syslog_prefix="", skip_prefix=0 ):
    print_user("Checking contents of output database %s, table: %s" % (dbname, tablename))
    return check_reader_expected(sql_reader((dbname, tablename)), messages, settle_time, syslog_prefix, skip_prefix)



#!/usr/bin/env python
#############################################################################
# Copyright (c) 2020 One Identity
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 2 as published
# by the Free Software Foundation, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#
# As an additional exemption you are allowed to compile & link against the
# OpenSSL libraries as published by the OpenSSL project. See the file
# COPYING for details.
#
#############################################################################
import pytest

from src.helpers.snmptrapd.conftest import *  # noqa:F403, F401


@pytest.mark.snmp
def test_snmp_dest_v2c_using_macros_in_snmp_obj(config, syslog_ng, snmptrapd, snmp_test_params):
    default_facility = "auth"
    default_severity = "0"
    program = "testprogram"
    message = "test message"
    input_message = "<38>Feb 11 21:27:22 testhost {}[9999]: {}\n".format(program, message)

    file_source = config.create_file_source(file_name="input.log")

    snmp_destination = config.create_snmp_destination(
        host=snmp_test_params.get_ip_address(),
        port=snmptrapd.get_port(),
        snmp_obj=(
            '"1.3.6.1.4.1.9.9.41.1.2.3.1.2.55", "Octetstring", "$FACILITY"',
            '"1.3.6.1.4.1.9.9.41.1.2.3.1.3.55", "Integer", "$SEVERITY"',
            '"1.3.6.1.4.1.9.9.41.1.2.3.1.4.55", "Octetstring", "$PROGRAM"',
            '"1.3.6.1.4.1.9.9.41.1.2.3.1.5.55", "Octetstring", "$MESSAGE"',
        ),
        trap_obj=snmp_test_params.get_cisco_trap_obj(),
        version="v2c",
    )
    config.create_logpath(statements=[file_source, snmp_destination])
    file_source.write_log(input_message)

    syslog_ng.start(config)

    expected_traps = [
        '.1.3.6.1.4.1.9.9.41.1.2.3.1.2.55 = STRING: "{}"'.format(default_facility),
        '.1.3.6.1.4.1.9.9.41.1.2.3.1.3.55 = INTEGER: {}'.format(default_severity),
        '.1.3.6.1.4.1.9.9.41.1.2.3.1.4.55 = STRING: "{}"'.format(program),
        '.1.3.6.1.4.1.9.9.41.1.2.3.1.5.55 = STRING: "{}"'.format(message),
        '.1.3.6.1.6.3.1.1.4.1.0 = OID: .1.3.6.1.4.1.9.9.41.2.0.1',
    ]

    assert expected_traps == snmptrapd.get_traps(len(expected_traps))



#!/usr/bin/env python
#############################################################################
# Copyright (c) 2021 One Identity
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 2 as published
# by the Free Software Foundation, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#
# As an additional exemption you are allowed to compile & link against the
# OpenSSL libraries as published by the OpenSSL project. See the file
# COPYING for details.
#
#############################################################################

_Input_String = "American Express  378282246310005" \
                "American Express  371449635398431" \
                "American Express Corporate  378734493671000" \
                "Diners Club  30569309025904" \
                "Diners Club  38520000023237" \
                "Discover  6011111111111117" \
                "Discover  6011000990139424" \
                "JCB  3530111333300000" \
                "JCB  3566002020360505" \
                "MasterCard  5555555555554444" \
                "MasterCard  5105105105105100" \
                "Visa  4111111111111111" \
                "Visa  4012888888881881" \
                "Visa  4222222222222"

_Expected_mask_output = "American Express  378282******0005" \
                        "American Express  371449******8431" \
                        "American Express Corporate  378734******1000" \
                        "Diners Club  305693******5904" \
                        "Diners Club  385200******3237" \
                        "Discover  601111******1117" \
                        "Discover  601100******9424" \
                        "JCB  353011******0000" \
                        "JCB  356600******0505" \
                        "MasterCard  555555******4444" \
                        "MasterCard  510510******5100" \
                        "Visa  411111******1111" \
                        "Visa  401288******1881" \
                        "Visa  422222******2222"

_Expected_hash_output = "American Express  ea4654336c140e70" \
                        "American Express  5e7d7549d9a51a21" \
                        "American Express Corporate  b83feb75b1ce505d" \
                        "Diners Club  58b3e8b7f99a5ab1" \
                        "Diners Club  002f83eefd0b7e53" \
                        "Discover  0ccaaf4da33d3e26" \
                        "Discover  ff659bd8ffefdb2b" \
                        "JCB  4c1d57bdab8338e7" \
                        "JCB  9d9cafd187ba5590" \
                        "MasterCard  6589b0d46b6f2f0d" \
                        "MasterCard  21b95eabb14f0726" \
                        "Visa  68bfb396f35af387" \
                        "Visa  62163a017b168ad4" \
                        "Visa  eb0f3622c9362fe9"


def test_cc_mask_and_cc_hash(config, syslog_ng):
    config.add_include("scl.conf")
    rewrite_cc_mask = config.create_rewrite_credit_card_mask()
    rewrite_cc_hash = config.create_rewrite_credit_card_hash()

    generator_source = config.create_example_msg_generator_source(num=1, template=config.stringify(_Input_String))
    file_destination_mask = config.create_file_destination(file_name="output_mask.log", template=config.stringify('${MSG}\n'))
    file_destination_hash = config.create_file_destination(file_name="output_hash.log", template=config.stringify('${MSG}\n'))
    config.create_logpath(statements=[generator_source, rewrite_cc_hash, file_destination_hash])
    config.create_logpath(statements=[generator_source, rewrite_cc_mask, file_destination_mask])

    syslog_ng.start(config)
    assert file_destination_mask.read_log().strip() == _Expected_mask_output
    assert file_destination_hash.read_log().strip() == _Expected_hash_output



#!/usr/bin/env python
#############################################################################
# Copyright (c) 2020 One Identity
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 2 as published
# by the Free Software Foundation, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#
# As an additional exemption you are allowed to compile & link against the
# OpenSSL libraries as published by the OpenSSL project. See the file
# COPYING for details.
#
#############################################################################

PROXY_PROTO_HEADER = "PROXY TCP4 192.168.1.1 192.168.1.2 20000 20001\r\n"
RFC3164_EXAMPLE = "<34>Oct 11 22:14:15 mymachine su: 'su root' failed for lonvick on /dev/pts/8\n"
RFC3164_EXAMPLE_WITHOUT_PRI = "Oct 11 22:14:15 mymachine su: 'su root' failed for lonvick on /dev/pts/8\n"


def test_pp_with_syslog_proto(config, port_allocator, syslog_ng, loggen):
    network_source = config.create_network_source(ip="localhost", port=port_allocator(), transport="proxied-tcp")
    file_destination = config.create_file_destination(file_name="output.log")
    config.create_logpath(statements=[network_source, file_destination])
    config.update_global_options(keep_hostname="yes")

    syslog_ng.start(config)

    network_source.write_log(PROXY_PROTO_HEADER + RFC3164_EXAMPLE)

    assert file_destination.read_log() == RFC3164_EXAMPLE_WITHOUT_PRI



#############################################################################
# Copyright (c) 2015-2016 Balabit
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 2 as published
# by the Free Software Foundation, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#
# As an additional exemption you are allowed to compile & link against the
# OpenSSL libraries as published by the OpenSSL project. See the file
# COPYING for details.
#
#############################################################################

from __future__ import absolute_import, print_function
from ..templatelang import TemplateLang
from .test_completerlang import CompleterLangTestCase


class TestTemplateLang(CompleterLangTestCase):
    def setUp(self):
        self._parser = TemplateLang()

    def test_template_is_a_series_of_literal_macro_and_template_funcs(self):
        self._assert_token_follows("", ["LITERAL", "MACRO", "TEMPLATE_FUNC"])
        self._assert_token_follows(" ", ["LITERAL", "MACRO", "TEMPLATE_FUNC"])
        self._assert_token_follows("foo ", ["LITERAL", "MACRO", "TEMPLATE_FUNC"])
        self._assert_token_follows("$MSG ", ["LITERAL", "MACRO", "TEMPLATE_FUNC"])
        self._assert_token_follows("${MSG} ", ["LITERAL", "MACRO", "TEMPLATE_FUNC"])
        self._assert_token_follows("$(echo $MSG) ", ["LITERAL", "MACRO", "TEMPLATE_FUNC"])
        self._assert_token_follows("foo$MSG${MSG}$(echo $MSG)bar ", ["LITERAL", "MACRO", "TEMPLATE_FUNC"])



#!/usr/bin/env python
#############################################################################
# Copyright (c) 2015-2018 Balabit
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 2 as published
# by the Free Software Foundation, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#
# As an additional exemption you are allowed to compile & link against the
# OpenSSL libraries as published by the OpenSSL project. See the file
# COPYING for details.
#
#############################################################################
import shlex

from src.common.random_id import get_unique_id
from src.executors.command_executor import CommandExecutor
from src.executors.process_executor import ProcessExecutor


class SyslogNgExecutor(object):
    def __init__(self, instance_paths):
        self.__instance_paths = instance_paths
        self.__process_executor = ProcessExecutor()
        self.__command_executor = CommandExecutor()

    def run_process(self, stderr, debug, trace, verbose, startup_debug, no_caps, config_path, persist_path, pid_path, control_socket_path):
        return self.__process_executor.start(
            command=self.__construct_syslog_ng_process(stderr, debug, trace, verbose, startup_debug, no_caps, config_path, persist_path, pid_path, control_socket_path),
            stdout_path=self.__instance_paths.get_stdout_path(),
            stderr_path=self.__instance_paths.get_stderr_path(),
        )

    def run_process_with_external_tool(self, external_tool, stderr, debug, trace, verbose, startup_debug, no_caps, config_path, persist_path, pid_path, control_socket_path):
        self.__instance_paths.register_external_tool_output_path(external_tool)
        if external_tool == "valgrind":
            return self.run_process_with_valgrind(stderr, debug, trace, verbose, startup_debug, no_caps, config_path, persist_path, pid_path, control_socket_path)
        elif external_tool == "strace":
            return self.run_process_with_strace(stderr, debug, trace, verbose, startup_debug, no_caps, config_path, persist_path, pid_path, control_socket_path)
        elif external_tool == "gdb":
            return self.run_process_with_gdb(stderr, debug, trace, verbose, startup_debug, no_caps, config_path, persist_path, pid_path, control_socket_path)
        else:
            raise Exception("Unknown external tool was selected: {}".format(external_tool))

    def run_process_with_valgrind(self, stderr, debug, trace, verbose, startup_debug, no_caps, config_path, persist_path, pid_path, control_socket_path):
        valgrind_command_args = [
            "valgrind",
            "--show-leak-kinds=all",
            "--track-origins=yes",
            "--tool=memcheck",
            "--leak-check=full",
            "--keep-stacktraces=alloc-and-free",
            "--read-var-info=yes",
            "--error-limit=no",
            "--num-callers=40",
            "--verbose",
            "--log-file={}".format(self.__instance_paths.get_external_tool_output_path("valgrind")),
        ]
        full_command_args = valgrind_command_args + self.__construct_syslog_ng_process(stderr, debug, trace, verbose, startup_debug, no_caps, config_path, persist_path, pid_path, control_socket_path)
        return self.__process_executor.start(
            command=full_command_args,
            stdout_path=self.__instance_paths.get_stdout_path(),
            stderr_path=self.__instance_paths.get_stderr_path(),
        )

    def run_process_with_gdb(self, stderr, debug, trace, verbose, startup_debug, no_caps, config_path, persist_path, pid_path, control_socket_path):
        syslog_ng_bin, *syslog_ng_args = self.__construct_syslog_ng_process(stderr, debug, trace, verbose, startup_debug, no_caps, config_path, persist_path, pid_path, control_socket_path)
        gdb_command_args = [
            "gdb",
            "-ex", "r {} > {} 2> {}".format(shlex.join(syslog_ng_args), self.__instance_paths.get_stdout_path(), self.__instance_paths.get_stderr_path()),
            syslog_ng_bin,
        ]
        return self.__process_executor.start(
            command=["xterm", "-e", shlex.join(gdb_command_args)],
            stdout_path="/dev/null",
            stderr_path="/dev/null",
        )

    def run_process_with_strace(self, stderr, debug, trace, verbose, startup_debug, no_caps, config_path, persist_path, pid_path, control_socket_path):
        strace_command_args = [
            "strace",
            "-s",
            "4096",
            "-tt",
            "-T",
            "-ff",
            "-o",
            self.__instance_paths.get_external_tool_output_path("strace"),
        ]
        full_command_args = strace_command_args + self.__construct_syslog_ng_process(stderr, debug, trace, verbose, startup_debug, no_caps, config_path, persist_path, pid_path, control_socket_path)
        return self.__process_executor.start(
            command=full_command_args,
            stdout_path=self.__instance_paths.get_stdout_path(),
            stderr_path=self.__instance_paths.get_stderr_path(),
        )

    def run_command(self, command_short_name, command):
        return self.__command_executor.run(
            command=self.__construct_syslog_ng_command(command),
            stdout_path=self.__instance_paths.get_stdout_path_with_postfix(postfix=command_short_name),
            stderr_path=self.__instance_paths.get_stderr_path_with_postfix(postfix=command_short_name),
        )

    def get_backtrace_from_core(self, core_file):
        gdb_command_args = [
            "gdb",
            "-ex",
            "bt full",
            "--batch",
            self.__instance_paths.get_syslog_ng_bin(),
            "--core",
            core_file,
        ]
        core_postfix = "gdb_core_{}".format(get_unique_id())
        return self.__command_executor.run(
            command=gdb_command_args,
            stdout_path=self.__instance_paths.get_stdout_path_with_postfix(postfix=core_postfix),
            stderr_path=self.__instance_paths.get_stderr_path_with_postfix(postfix=core_postfix),
        )

    def __construct_syslog_ng_process(
        self,
        stderr,
        debug,
        trace,
        verbose,
        startup_debug,
        no_caps,
        config_path,
        persist_path,
        pid_path,
        control_socket_path,
    ):
        syslog_ng_process_args = [str(self.__instance_paths.get_syslog_ng_bin())]
        syslog_ng_process_args += ["--foreground", "--enable-core"]
        if stderr:
            syslog_ng_process_args += ["--stderr"]
        if debug:
            syslog_ng_process_args += ["--debug"]
        if trace:
            syslog_ng_process_args += ["--trace"]
        if verbose:
            syslog_ng_process_args += ["--verbose"]
        if startup_debug:
            syslog_ng_process_args += ["--startup-debug"]
        if no_caps:
            syslog_ng_process_args += ["--no-caps"]
        if config_path is None:
            config_path = self.__instance_paths.get_config_path()
        syslog_ng_process_args += ["--cfgfile={}".format(config_path)]
        if persist_path is None:
            persist_path = self.__instance_paths.get_persist_path()
        syslog_ng_process_args += ["--persist-file={}".format(persist_path)]
        if pid_path is None:
            pid_path = self.__instance_paths.get_pid_path()
        syslog_ng_process_args += ["--pidfile={}".format(pid_path)]
        if control_socket_path is None:
            control_socket_path = self.__instance_paths.get_control_socket_path()
        syslog_ng_process_args += ["--control={}".format(control_socket_path)]
        return syslog_ng_process_args

    def __construct_syslog_ng_command(self, command):
        syslog_ng_command_args = [self.__instance_paths.get_syslog_ng_bin()]
        syslog_ng_command_args += command
        return syslog_ng_command_args



#!/usr/bin/env python
#############################################################################
# Copyright (c) 2015-2019 Balabit
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 2 as published
# by the Free Software Foundation, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#
# As an additional exemption you are allowed to compile & link against the
# OpenSSL libraries as published by the OpenSSL project. See the file
# COPYING for details.
#
#############################################################################


def test_flags_catch_all(config, syslog_ng, log_message, bsd_formatter):
    # Check the correct output if the logpath is the following
    # log {
    #     source(s_file);
    #     log { destination(d_file1); };
    # };
    # log { destination(d_file2); flags(catch-all);};
    # input logs:
    # Oct 11 22:14:15 host-A testprogram: message from host-A

    file_source = config.create_file_source(file_name="input.log")
    file_destination = config.create_file_destination(file_name="output.log")
    catch_all_destination = config.create_file_destination(file_name="catchall_output.log")

    inner_logpath = config.create_inner_logpath(statements=[file_destination])

    config.create_logpath(statements=[file_source, inner_logpath])
    config.create_logpath(statements=[catch_all_destination], flags="catch-all")
    config.update_global_options(keep_hostname="yes")

    input_message = bsd_formatter.format_message(log_message)
    expected_message = bsd_formatter.format_message(log_message.remove_priority())
    file_source.write_log(input_message)

    syslog_ng.start(config)

    destination_log = file_destination.read_log()
    # message should arrived into destination1
    assert expected_message in destination_log

    catch_all_destination_log = catch_all_destination.read_log()
    # message should arrived into catch_all_destination
    # there is a flags(catch-all)
    assert expected_message in catch_all_destination_log



#!/usr/bin/env python
#############################################################################
# Copyright (c) 2015-2018 Balabit
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 2 as published
# by the Free Software Foundation, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#
# As an additional exemption you are allowed to compile & link against the
# OpenSSL libraries as published by the OpenSSL project. See the file
# COPYING for details.
#
#############################################################################
from src.syslog_ng_ctl.syslog_ng_ctl_cli import SyslogNgCtlCli
from src.syslog_ng_ctl.syslog_ng_ctl_executor import QueryTypes


class SyslogNgCtl(object):
    def __init__(self, instance_paths):
        self.__syslog_ng_ctl_cli = SyslogNgCtlCli(instance_paths)

    def reload(self):
        return self.__syslog_ng_ctl_cli.reload()

    def stop(self):
        return self.__syslog_ng_ctl_cli.stop()

    def reopen(self):
        return self.__syslog_ng_ctl_cli.reopen()

    def stats(self, reset=False):
        return self.__syslog_ng_ctl_cli.stats(reset)

    def stats_prometheus(self):
        return self.__syslog_ng_ctl_cli.stats_prometheus()

    def query(self, pattern="*", query_type=QueryTypes.QUERY_GET):
        return self.__syslog_ng_ctl_cli.query(pattern, query_type)

    def credentials_add(self, credential, secret):
        return self.__syslog_ng_ctl_cli.credentials_add(credential, secret)

    def is_control_socket_alive(self):
        return self.__syslog_ng_ctl_cli.is_control_socket_alive()



#!/usr/bin/env python
#############################################################################
# Copyright (c) 2020 One Identity
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 2 as published
# by the Free Software Foundation, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#
# As an additional exemption you are allowed to compile & link against the
# OpenSSL libraries as published by the OpenSSL project. See the file
# COPYING for details.
#
#############################################################################
from pathlib import Path

from psutil import TimeoutExpired

import src.testcase_parameters.testcase_parameters as tc_parameters
from src.executors.process_executor import ProcessExecutor


class Loggen(object):
    instanceIndex = -1

    @staticmethod
    def __get_new_instance_index():
        Loggen.instanceIndex += 1
        return Loggen.instanceIndex

    def __init__(self):
        self.loggen_proc = None
        self.loggen_bin_path = tc_parameters.INSTANCE_PATH.get_loggen_bin()

    def __decode_start_parameters(
            self, inet, unix, stream, dgram, use_ssl, dont_parse, read_file, skip_tokens, loop_reading,
            rate, interval, permanent, syslog_proto, proxied, sdata, no_framing, active_connections,
            idle_connections, ipv6, debug, number, csv, quiet, size, reconnect, proxied_tls_passthrough,
            proxy_src_ip, proxy_dst_ip, proxy_src_port, proxy_dst_port,
    ):

        start_parameters = []

        if inet is True:
            start_parameters.append("--inet")

        if unix is True:
            start_parameters.append("--unix")

        if stream is True:
            start_parameters.append("--stream")

        if dgram is True:
            start_parameters.append("--dgram")

        if use_ssl is True:
            start_parameters.append("--use-ssl")

        if dont_parse is True:
            start_parameters.append("--dont-parse")

        if read_file is not None:
            start_parameters.append("--read-file={}".format(read_file))

        if skip_tokens is not None:
            start_parameters.append("--skip-tokens={}".format(skip_tokens))

        if loop_reading is True:
            start_parameters.append("--loop-reading")

        if rate is not None:
            start_parameters.append("--rate={}".format(int(rate)))

        if interval is not None:
            start_parameters.append("--interval={}".format(interval))

        if permanent is True:
            start_parameters.append("--permanent")

        if syslog_proto is True:
            start_parameters.append("--syslog-proto")

        if proxied is True:
            start_parameters.append("--proxied")

        if proxied == 1 or proxied == 2:
            start_parameters.append("--proxied={}".format(proxied))

        if proxy_src_ip is not None:
            start_parameters.append("--proxy-src-ip={}".format(proxy_src_ip))

        if proxy_dst_ip is not None:
            start_parameters.append("--proxy-dst-ip={}".format(proxy_dst_ip))

        if proxy_src_port is not None:
            start_parameters.append("--proxy-src-port={}".format(proxy_src_port))

        if proxy_dst_port is not None:
            start_parameters.append("--proxy-dst-port={}".format(proxy_dst_port))

        if proxied_tls_passthrough is True:
            start_parameters.append("--proxied-tls-passthrough")

        if sdata is True:
            start_parameters.append("--sdata")

        if no_framing is True:
            start_parameters.append("--no-framing")

        if active_connections is not None:
            start_parameters.append("--active-connections={}".format(active_connections))

        if idle_connections is not None:
            start_parameters.append("--idle-connections={}".format(idle_connections))

        if ipv6 is True:
            start_parameters.append("--ipv6")

        if debug is True:
            start_parameters.append("--debug")

        if number is not None:
            start_parameters.append("--number={}".format(number))

        if csv is True:
            start_parameters.append("--csv")

        if quiet is True:
            start_parameters.append("--quiet")

        if size is not None:
            start_parameters.append("--size={}".format(size))

        if reconnect is True:
            start_parameters.append("--reconnect")

        return start_parameters

    def start(
            self, target, port, inet=None, unix=None, stream=None, dgram=None, use_ssl=None, dont_parse=None,
            read_file=None, skip_tokens=None, loop_reading=None,
            rate=None, interval=None, permanent=None, syslog_proto=None, proxied=None, sdata=None, no_framing=None,
            active_connections=None,
            idle_connections=None, ipv6=None, debug=None, number=None, csv=None, quiet=None, size=None, reconnect=None,
            proxied_tls_passthrough=None,
            proxy_src_ip=None, proxy_dst_ip=None, proxy_src_port=None, proxy_dst_port=None,
    ):

        if self.loggen_proc is not None and self.loggen_proc.is_running():
            raise Exception("Loggen is already running, you shouldn't call start")

        instanceIndex = Loggen.__get_new_instance_index()
        self.loggen_stdout_path = Path("loggen_stdout_{}".format(instanceIndex))
        self.loggen_stderr_path = Path("loggen_stderr_{}".format(instanceIndex))

        self.parameters = self.__decode_start_parameters(
            inet, unix, stream, dgram, use_ssl, dont_parse, read_file, skip_tokens, loop_reading,
            rate, interval, permanent, syslog_proto, proxied, sdata, no_framing, active_connections,
            idle_connections, ipv6, debug, number, csv, quiet, size, reconnect, proxied_tls_passthrough,
            proxy_src_ip, proxy_dst_ip, proxy_src_port, proxy_dst_port,
        )

        self.loggen_proc = ProcessExecutor().start(
            [self.loggen_bin_path] + self.parameters + [target, port],
            self.loggen_stdout_path,
            self.loggen_stderr_path,
        )

        return self.loggen_proc

    def stop(self):
        if self.loggen_proc is None:
            return

        self.loggen_proc.terminate()
        try:
            self.loggen_proc.wait(4)
        except TimeoutExpired:
            self.loggen_proc.kill()

        self.loggen_proc = None

    def get_sent_message_count(self):
        if not self.loggen_stderr_path.exists():
            return 0

        # loggen puts the count= messages to the stderr
        f = open(self.loggen_stderr_path, "r")
        content = f.read()
        f.close()

        start_pattern = "count="
        if start_pattern not in content:
            return 0
        index_start = content.rindex(start_pattern) + len(start_pattern)

        end_pattern = ", "
        if end_pattern not in content:
            return 0
        index_end = content.find(end_pattern, index_start)

        return int(content[index_start:index_end])



#!/usr/bin/env python
#############################################################################
# Copyright (c) 2021 One Identity
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 2 as published
# by the Free Software Foundation, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#
# As an additional exemption you are allowed to compile & link against the
# OpenSSL libraries as published by the OpenSSL project. See the file
# COPYING for details.
#
#############################################################################
from src.message_builder.bsd_format import BSDFormat
from src.message_builder.log_message import LogMessage


def test_filter_multiple_reference(config, syslog_ng):
    config.update_global_options(keep_hostname="yes")

    file_source = config.create_file_source(file_name="input.log")
    file_src_group = config.create_statement_group(file_source)
    negated_filter = config.create_statement_group(config.create_filter("not (program('noprog') and message('nomsg'))"))
    file_destination1 = config.create_file_destination(file_name="output1.log")
    file_destination2 = config.create_file_destination(file_name="output2.log")

    config.create_logpath(statements=[file_src_group, negated_filter, file_destination1])
    config.create_logpath(statements=[file_src_group, negated_filter, file_destination2])

    log_msg = LogMessage().program("PROGRAM").message("MESSAGE")
    bsd_msg = BSDFormat.format_message(log_msg.remove_priority())

    file_source.write_log(bsd_msg)

    syslog_ng.start(config)

    dest1_logs = file_destination1.read_logs(counter=1)
    assert bsd_msg in dest1_logs

    dest2_logs = file_destination2.read_logs(counter=1)
    assert bsd_msg in dest2_logs



#############################################################################
# Copyright (c) 2015-2016 Balabit
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 2 as published
# by the Free Software Foundation, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#
# As an additional exemption you are allowed to compile & link against the
# OpenSSL libraries as published by the OpenSSL project. See the file
# COPYING for details.
#
#############################################################################

from __future__ import absolute_import, print_function
import unittest
from ..debuglang import DebugLang


class CompleterLangTestCase(unittest.TestCase):
    def _get_expected_tokens(self, text):
        drop_last_token = not text[-1:].isspace()
        return self._parser.get_expected_tokens(text, drop_last_token=drop_last_token)

    def _assert_token_follows(self, text, tokens, pos=None, replaced_token=None):
        (expected_tokens, rtoken, rtoken_pos) = self._get_expected_tokens(text)
        if pos is not None:
            self.assertEqual(pos, rtoken_pos)
        if replaced_token is not None:
            self.assertEqual(rtoken.value if rtoken is not None else '', replaced_token)
        self.assertLessEqual(set(tokens), set(expected_tokens))


class TestCompleterLang(CompleterLangTestCase):
    """This testcase intends to test the CompleterLang base class by
    an example grammar, which happens to be DebugLang. The DebugLang grammar
    itself is tested in a separate testcase."""

    def setUp(self):
        self._parser = DebugLang()

    def test_all_tokens_are_found_that_would_match_rules_in_the_grammar(self):
        self._assert_command_token_follows("", pos=0, replaced_token="")
        self._assert_command_token_follows(" ", pos=1, replaced_token="")
        self._assert_command_token_follows("foo", pos=0, replaced_token="foo")

        self._assert_token_follows("foo ", ["ARG"], pos=4, replaced_token="")
        self._assert_token_follows("foo bar", ["ARG"], pos=4, replaced_token="bar")
        self._assert_token_follows("foo bar ", ["ARG"], pos=8, replaced_token="")
        self._assert_token_follows("print ", ["template", "ARG"], pos=6, replaced_token="")
        self._assert_token_follows("print $MSG", ["template", "ARG"], pos=6, replaced_token="$MSG")
        self._assert_token_follows("print $MSG ", [], pos=11, replaced_token="")

    def _get_command_tokens(self):
        return [token for token in self._parser.tokens if token.startswith("COMMAND")]

    def _assert_command_token_follows(self, text, pos=None, replaced_token=None):
        self._assert_token_follows(text, set(self._get_command_tokens()), pos, replaced_token)



#!/usr/bin/env python
#############################################################################
# Copyright (c) 2020 Balabit
# Copyright (c) 2020 Nobles
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 2 as published
# by the Free Software Foundation, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#
# As an additional exemption you are allowed to compile & link against the
# OpenSSL libraries as published by the OpenSSL project. See the file
# COPYING for details.
#
#############################################################################
import pytest

DEFAULT_PID = "pid"


@pytest.mark.parametrize(
    "use_syslogng_pid", [("yes", "no")],
)
def test_use_syslogng_pid(config, syslog_ng, use_syslogng_pid):
    generator_source = config.create_example_msg_generator_source(num=1, use_syslogng_pid=use_syslogng_pid, values="PID => {}".format(DEFAULT_PID))
    file_destination = config.create_file_destination(file_name="output.log", template=config.stringify("PID=$PID\n"))
    config.create_logpath(statements=[generator_source, file_destination])

    p = syslog_ng.start(config)
    if use_syslogng_pid == "yes":
        expected_value = "PID={}".format(p.pid)
    else:
        expected_value = "PID={}".format(DEFAULT_PID)
    assert file_destination.read_log().strip() == expected_value



#############################################################################
# Copyright (c) 2007-2015 Balabit
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 2 as published
# by the Free Software Foundation, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#
# As an additional exemption you are allowed to compile & link against the
# OpenSSL libraries as published by the OpenSSL project. See the file
# COPYING for details.
#
#############################################################################

from globals import *
from log import *
from messagegen import *
from messagecheck import *
from control import flush_files, stop_syslogng

config = """@version: %(syslog_ng_version)s

options { ts_format(iso); chain_hostnames(no); keep_hostname(yes); threaded(yes); };

source s_int { internal(); };
source s_tcp { tcp(port(%(port_number)d)); };

destination d_sql {
    sql(type(sqlite3) database("%(current_dir)s/test-sql.db") host(dummy) port(1234) username(dummy) password(dummy)
        table("logs")
        null("@NULL@")
        columns("date datetime", "host", "program", "pid", "msg", "dummy int default 5678")
        values("$DATE", "$HOST", "$PROGRAM", "${PID:-@NULL@}", "$MSG", default)
        indexes("date", "host", "program")
        flags(explicit-commits)
        flush-lines(25) flush_timeout(100));
};

log { source(s_tcp); destination(d_sql); };

""" % locals()

def check_env():

    if not has_module('afsql'):
        print('afsql module is not available, skipping SQL test')
        return False
    paths=('/opt/syslog-ng/bin', '/usr/bin', '/usr/local/bin')
    found=False
    for pth in paths:
        if os.path.isfile(os.path.join(pth, 'sqlite3')):
            found = True
    if not found:
        print_user("no sqlite3 tool, skipping SQL test\nSearched: %s\n" % ':'.join(paths))
        return False

    soext='.so'
    if re.match('hp-ux', sys.platform) and not re.match('ia64', os.uname()[4]):
        soext='.sl'

    found = False
    paths = (os.environ.get('dbd_dir', ''), '/usr/local/lib/dbd', '/usr/lib/dbd',
        '/usr/lib64/dbd/', '/opt/syslog-ng/lib/dbd', '/usr/lib/x86_64-linux-gnu/dbd')
    for pth in paths:
        if pth and os.path.isfile('%s/libdbdsqlite3%s' % (pth, soext)):
            found = True
            break
    if not found:
        print_user('No sqlite3 backend for libdbi. Skipping SQL test.\nSearched: %s\n' % ':'.join(paths))
        return False

    print('sqlite3 found, proceeding to SQL tests')
    return True


def test_sql():

    messages = (
        'sql1',
        'sql2'
    )
    s = SocketSender(AF_INET, ('localhost', port_number), dgram=0)

    expected = []
    for msg in messages:
        expected.extend(s.sendMessages(msg, pri=7))
    print_user("Waiting for 10 seconds until syslog-ng writes all records to the SQL table")
    time.sleep(10)
    stopped = stop_syslogng()
    time.sleep(5)
    return stopped and check_sql_expected("%s/test-sql.db" % current_dir, "logs", expected, settle_time=5, syslog_prefix="Sep  7 10:43:21 bzorp prog 12345")



#!/usr/bin/env python
#############################################################################
# Copyright (c) 2023 Balazs Scheidler <balazs.scheidler@axoflow.com>
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 2 as published
# by the Free Software Foundation, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#
# As an additional exemption you are allowed to compile & link against the
# OpenSSL libraries as published by the OpenSSL project. See the file
# COPYING for details.
#
#############################################################################
import json

import pytest

from src.syslog_ng_config.renderer import render_statement


def create_config(config, filterx_expr, msg="foobar"):
    file_true = config.create_file_destination(file_name="dest-true.log", template="'$MSG\n'")
    file_false = config.create_file_destination(file_name="dest-false.log", template="'$MSG\n'")

    preamble = f"""
@version: {config.get_version()}

options {{ stats(level(1)); }};

source genmsg {{
    example-msg-generator(
        num(1)
        template("{msg}")
        values(
            "values.str" => string("string"),
            "values.bool" => boolean(true),
            "values.int" => int(5),
            "values.double" => double(32.5),
            "values.datetime" => datetime("1701350398.123000+01:00"),
            "values.list" => list("foo,bar,baz"),
            "values.null" => null(""),
            "values.bytes" => bytes("binary whatever"),
            "values.protobuf" => protobuf("this is not a valid protobuf!!"),
            "values.json" => json('{{"emb_key1": "emb_key1 value", "emb_key2": "emb_key2 value"}}'),
            "values.true_string" => string("boolean:true"),
            "values.false_string" => string("boolean:false"),
        )
    );
}};

destination dest_true {{
    {render_statement(file_true)};
}};

destination dest_false {{
    {render_statement(file_false)};
}};

log {{
    source(genmsg);
    if {{
        filterx {{ {filterx_expr} \n}};
        destination(dest_true);
    }} else {{
        destination(dest_false);
    }};
}};
"""
    config.set_raw_config(preamble)
    return (file_true, file_false)

# TODO: take care of _evaluate_statement methods result, to fix this test
# def test_filterx_falsey_value_assign(config, syslog_ng):
#     (file_true, file_false) = create_config(
#         config, """
#                     $myvar = 0;
#                     $MSG = $myvar; """,
#     )
#     syslog_ng.start(config)

#     assert file_true.get_stats()["processed"] == 1
#     assert "processed" not in file_false.get_stats()
#     assert file_true.read_log() == "0\n"


def test_otel_logrecord_int32_setter_getter(config, syslog_ng):
    (file_true, file_false) = create_config(
        config, """
                                            $olr = otel_logrecord();
                                            $olr.dropped_attributes_count = ${values.int};
                                            $MSG = $olr.dropped_attributes_count; """,
    )
    syslog_ng.start(config)

    assert file_true.get_stats()["processed"] == 1
    assert "processed" not in file_false.get_stats()
    assert file_true.read_log() == "5\n"


def test_otel_logrecord_string_setter_getter(config, syslog_ng):
    (file_true, file_false) = create_config(
        config, """
                                            $olr = otel_logrecord();
                                            $olr.severity_text = "${values.str}";
                                            $MSG = $olr.severity_text; """,
    )
    syslog_ng.start(config)

    assert file_true.get_stats()["processed"] == 1
    assert "processed" not in file_false.get_stats()
    assert file_true.read_log() == "string\n"


def test_otel_logrecord_bytes_setter_getter(config, syslog_ng):
    (file_true, file_false) = create_config(
        config, """
                                            $olr = otel_logrecord();
                                            $olr.trace_id = ${values.bytes};
                                            $MSG = $olr.trace_id; """,
    )
    syslog_ng.start(config)

    assert file_true.get_stats()["processed"] == 1
    assert "processed" not in file_false.get_stats()
    assert file_true.read_log() == "binary whatever\n"


def test_otel_logrecord_datetime_setter_getter(config, syslog_ng):
    (file_true, file_false) = create_config(
        config, """
                                            $olr = otel_logrecord();
                                            $olr.time_unix_nano = ${values.datetime};
                                            $MSG = $olr.time_unix_nano; """,
    )
    syslog_ng.start(config)

    assert file_true.get_stats()["processed"] == 1
    assert "processed" not in file_false.get_stats()
    assert file_true.read_log() == "1701353998.123000+00:00\n"


def test_otel_logrecord_body_string_setter_getter(config, syslog_ng):
    (file_true, file_false) = create_config(
        config, """
                                            $olr = otel_logrecord();
                                            $olr.body = ${values.str};
                                            $MSG = $olr.body; """,
    )
    syslog_ng.start(config)

    assert file_true.get_stats()["processed"] == 1
    assert "processed" not in file_false.get_stats()
    assert file_true.read_log() == "string\n"


def test_otel_logrecord_body_bool_setter_getter(config, syslog_ng):
    (file_true, file_false) = create_config(
        config, """
                                            $olr = otel_logrecord();
                                            $olr.body = ${values.bool};
                                            $MSG = $olr.body; """,
    )
    syslog_ng.start(config)

    assert file_true.get_stats()["processed"] == 1
    assert "processed" not in file_false.get_stats()
    assert file_true.read_log() == "true\n"


def test_otel_logrecord_body_int_setter_getter(config, syslog_ng):
    (file_true, file_false) = create_config(
        config, """
                                            $olr = otel_logrecord();
                                            $olr.body = ${values.int};
                                            $MSG = $olr.body; """,
    )
    syslog_ng.start(config)

    assert file_true.get_stats()["processed"] == 1
    assert "processed" not in file_false.get_stats()
    assert file_true.read_log() == "5\n"


def test_otel_logrecord_body_double_setter_getter(config, syslog_ng):
    (file_true, file_false) = create_config(
        config, """
                                            $olr = otel_logrecord();
                                            $olr.body = ${values.double};
                                            $MSG = $olr.body; """,
    )
    syslog_ng.start(config)

    assert file_true.get_stats()["processed"] == 1
    assert "processed" not in file_false.get_stats()
    assert file_true.read_log() == "32.5\n"


def test_otel_logrecord_body_datetime_setter_getter(config, syslog_ng):
    (file_true, file_false) = create_config(
        config, """
                                            $olr = otel_logrecord();
                                            $olr.body = ${values.datetime};
                                            $MSG = $olr.body; """,
    )
    syslog_ng.start(config)

    # there is no implicit conversion back to datetime because anyvalue field
    # does not distinguish int_value and datetime.
    assert file_true.get_stats()["processed"] == 1
    assert "processed" not in file_false.get_stats()
    assert file_true.read_log() == "1701353998123000\n"


# TODO: figure out null type's proper behaviour
# def test_otel_logrecord_body_null_setter_getter(config, syslog_ng):
#     (file_true, file_false) = create_config(
#         config, """
#                                             $olr = otel_logrecord();
#                                             $olr.body = ${values.null};
#                                             $MSG = $olr.body; """,
#     )
#     syslog_ng.start(config)

#     assert file_true.get_stats()["processed"] == 1
#     assert "processed" not in file_false.get_stats()
#     assert file_true.read_log() == "\n"


def test_otel_logrecord_body_bytes_setter_getter(config, syslog_ng):
    (file_true, file_false) = create_config(
        config, """
                                            $olr = otel_logrecord();
                                            $olr.body = ${values.bytes};
                                            $MSG = $olr.body; """,
    )
    syslog_ng.start(config)

    assert file_true.get_stats()["processed"] == 1
    assert "processed" not in file_false.get_stats()
    assert file_true.read_log() == "binary whatever\n"


def test_otel_logrecord_body_protobuf_setter_getter(config, syslog_ng):
    (file_true, file_false) = create_config(
        config, """
                                            $olr = otel_logrecord();
                                            $olr.body = ${values.protobuf};
                                            $MSG = $olr.body; """,
    )
    syslog_ng.start(config)

    assert file_true.get_stats()["processed"] == 1
    assert "processed" not in file_false.get_stats()
    assert file_true.read_log() == "this is not a valid protobuf!!\n"


def test_otel_logrecord_body_json_setter_getter(config, syslog_ng):
    (file_true, file_false) = create_config(
        config, """
                                            $olr = otel_logrecord();
                                            $olr.body = ${values.json};
                                            istype($olr.body, "otel_kvlist");
                                            $MSG = format_json($olr.body); """,
    )
    syslog_ng.start(config)

    assert file_true.get_stats()["processed"] == 1
    assert "processed" not in file_false.get_stats()
    assert file_true.read_log() == '{"emb_key1":"emb_key1 value","emb_key2":"emb_key2 value"}\n'


def test_json_to_otel(config, syslog_ng):
    (file_true, file_false) = create_config(
        config, r"""
                                            js = json({"foo": 42});
                                            js_arr = json_array([1, 2]);

                                            otel_kvl = otel_kvlist();

                                            otel_kvl.js = js;
                                            istype(otel_kvl.js, "otel_kvlist");
                                            otel_kvl.js += {"bar": 1337};

                                            otel_kvl.js_arr = js_arr;
                                            istype(otel_kvl.js_arr, "otel_array");
                                            otel_kvl.js_arr += [3, 4];

                                            $MSG = format_json(otel_kvl);
                """,
    )
    syslog_ng.start(config)

    assert file_true.get_stats()["processed"] == 1
    assert "processed" not in file_false.get_stats()
    assert file_true.read_log() == '{"js":{"foo":42,"bar":1337},"js_arr":[1,2,3,4]}\n'


def test_otel_to_json(config, syslog_ng):
    (file_true, file_false) = create_config(
        config, r"""
                                            otel_kvl = otel_kvlist({"foo": 42});
                                            otel_arr = otel_array([1, 2]);

                                            js = json();

                                            js.otel_kvl = otel_kvl;
                                            istype(js.otel_kvl, "json_object");
                                            js.otel_kvl += {"bar": 1337};

                                            js.otel_arr = otel_arr;
                                            istype(js.otel_arr, "json_array");
                                            js.otel_arr += [3, 4];

                                            $MSG = js;
                """,
    )
    syslog_ng.start(config)

    assert file_true.get_stats()["processed"] == 1
    assert "processed" not in file_false.get_stats()
    assert file_true.read_log() == '{"otel_kvl":{"foo":42,"bar":1337},"otel_arr":[1,2,3,4]}\n'


def test_simple_true_condition(config, syslog_ng):
    (file_true, file_false) = create_config(config, """ true; """)
    syslog_ng.start(config)

    assert file_true.get_stats()["processed"] == 1
    assert "processed" not in file_false.get_stats()
    assert file_true.read_log() == "foobar\n"


def test_simple_false_condition(config, syslog_ng):
    (file_true, file_false) = create_config(config, """ false; """)
    syslog_ng.start(config)

    assert "processed" not in file_true.get_stats()
    assert file_false.get_stats()["processed"] == 1
    assert file_false.read_log() == "foobar\n"


def test_simple_assignment(config, syslog_ng):

    (file_true, file_false) = create_config(config, """ $MSG = "rewritten message!"; """)
    syslog_ng.start(config)

    assert file_true.get_stats()["processed"] == 1
    assert "processed" not in file_false.get_stats()
    assert file_true.read_log() == "rewritten message!\n"


def test_json_assignment_from_template(config, syslog_ng):

    (file_true, file_false) = create_config(config, """ $MSG = "$(format-json --subkeys values.)"; """)
    syslog_ng.start(config)

    assert file_true.get_stats()["processed"] == 1
    assert "processed" not in file_false.get_stats()
    assert file_true.read_log() == """{"true_string":"boolean:true","str":"string","null":null,"list":["foo","bar","baz"],"json":{"emb_key1": "emb_key1 value", "emb_key2": "emb_key2 value"},"int":5,"false_string":"boolean:false","double":32.5,"datetime":"1701350398.123000+01:00","bool":true}\n"""


def test_json_assignment_from_another_name_value_pair(config, syslog_ng):

    (file_true, file_false) = create_config(config, """ $MSG = ${values.json}; """)
    syslog_ng.start(config)

    assert file_true.get_stats()["processed"] == 1
    assert "processed" not in file_false.get_stats()
    assert file_true.read_log() == """{"emb_key1": "emb_key1 value", "emb_key2": "emb_key2 value"}\n"""


def test_json_getattr_returns_the_embedded_json(config, syslog_ng):

    (file_true, file_false) = create_config(
        config, """
$envelope = "$(format-json --subkeys values.)";
$MSG = $envelope.json.emb_key1;
""",
    )
    syslog_ng.start(config)

    assert file_true.get_stats()["processed"] == 1
    assert "processed" not in file_false.get_stats()
    assert file_true.read_log() == """emb_key1 value\n"""


def test_json_setattr_sets_the_embedded_json(config, syslog_ng):

    (file_true, file_false) = create_config(
        config, """
$envelope = "$(format-json --subkeys values.)";
$envelope.json.new_key = "this is a new key!";
$MSG = $envelope.json;
""",
    )
    syslog_ng.start(config)

    assert file_true.get_stats()["processed"] == 1
    assert "processed" not in file_false.get_stats()
    assert file_true.read_log() == """{"emb_key1":"emb_key1 value","emb_key2":"emb_key2 value","new_key":"this is a new key!"}\n"""


def test_json_assign_performs_a_deep_copy(config, syslog_ng):

    (file_true, file_false) = create_config(
        config, """
$envelope = "$(format-json --subkeys values.)";
$envelope.json.new_key = "this is a new key!";
$MSG = $envelope.json;
$envelope.json.another_key = "this is another new key which is not added to $MSG!";
""",
    )
    syslog_ng.start(config)

    assert file_true.get_stats()["processed"] == 1
    assert "processed" not in file_false.get_stats()
    assert file_true.read_log() == """{"emb_key1":"emb_key1 value","emb_key2":"emb_key2 value","new_key":"this is a new key!"}\n"""


def test_json_simple_literal_assignment(config, syslog_ng):
    (file_true, file_false) = create_config(
        config, """
$MSG = json({
    "foo": "foovalue",
    "bar": "barvalue",
    "baz": "bazvalue"
});
""",
    )
    syslog_ng.start(config)

    assert file_true.get_stats()["processed"] == 1
    assert "processed" not in file_false.get_stats()
    assert file_true.read_log() == """{"foo":"foovalue","bar":"barvalue","baz":"bazvalue"}\n"""


def test_json_recursive_literal_assignment(config, syslog_ng):
    (file_true, file_false) = create_config(
        config, """
$MSG = json({
    "foo": "foovalue",
    "bar": "barvalue",
    "baz": "bazvalue",
    "recursive": {
        "foo": "foovalue",
        "bar": "barvalue",
        "baz": "bazvalue",
        "recursive": {
            "foo": "foovalue",
            "bar": "barvalue",
            "baz": "bazvalue",
        },
    },
});
""",
    )
    syslog_ng.start(config)

    assert file_true.get_stats()["processed"] == 1
    assert "processed" not in file_false.get_stats()
    assert file_true.read_log() == """{"foo":"foovalue","bar":"barvalue","baz":"bazvalue","recursive":{"foo":"foovalue","bar":"barvalue","baz":"bazvalue","recursive":{"foo":"foovalue","bar":"barvalue","baz":"bazvalue"}}}\n"""


def test_json_change_recursive_literal(config, syslog_ng):
    (file_true, file_false) = create_config(
        config, """
$MSG = json({
    "foo": "foovalue",
    "bar": "barvalue",
    "baz": "bazvalue",
    "recursive": {
        "foo": "foovalue",
        "bar": "barvalue",
        "baz": "bazvalue",
        "recursive": {
            "foo": "foovalue",
            "bar": "barvalue",
            "baz": "bazvalue",
        },
    },
});

$MSG.recursive.recursive.foo = "changedfoovalue";
$MSG.recursive.recursive.newattr = "newattrvalue";
""",
    )
    syslog_ng.start(config)

    assert file_true.get_stats()["processed"] == 1
    assert "processed" not in file_false.get_stats()
    assert file_true.read_log() == """\
{"foo":"foovalue","bar":"barvalue","baz":"bazvalue",\
"recursive":{"foo":"foovalue","bar":"barvalue","baz":"bazvalue",\
"recursive":{"foo":"changedfoovalue","bar":"barvalue","baz":"bazvalue","newattr":"newattrvalue"}}}\n"""


def test_list_literal_becomes_syslogng_list_as_string(config, syslog_ng):
    (file_true, file_false) = create_config(
        config, """
$MSG = json_array(["foo", "bar", "baz"]);
""",
    )
    syslog_ng.start(config)

    assert file_true.get_stats()["processed"] == 1
    assert "processed" not in file_false.get_stats()
    assert file_true.read_log() == """foo,bar,baz\n"""


def test_list_literal_becomes_json_list_as_a_part_of_json(config, syslog_ng):
    (file_true, file_false) = create_config(
        config, """
$list = json_array(["foo", "bar", "baz"]);
$MSG = json({
    "key": "value",
    "list": $list,
});
""",
    )
    syslog_ng.start(config)

    assert file_true.get_stats()["processed"] == 1
    assert "processed" not in file_false.get_stats()
    assert file_true.read_log() == """{"key":"value","list":["foo","bar","baz"]}\n"""


def test_list_is_cloned_upon_assignment(config, syslog_ng):
    (file_true, file_false) = create_config(
        config, """
$list = json_array(["foo", "bar", "baz"]);
$MSG = $list;
$list[0] = "changed foo";
$MSG[2] = "changed baz";
""",
    )
    syslog_ng.start(config)

    assert file_true.get_stats()["processed"] == 1
    assert "processed" not in file_false.get_stats()
    # foo remains unchanged while baz is changed
    assert file_true.read_log() == """foo,bar,"changed baz"\n"""


def test_list_subscript_without_index_appends_an_element(config, syslog_ng):
    (file_true, file_false) = create_config(
        config, """
$list = json_array();
$list[] = "foo";
$list[] = "bar";
$list[] = "baz";
$MSG = $list;
""",
    )
    syslog_ng.start(config)

    assert file_true.get_stats()["processed"] == 1
    assert "processed" not in file_false.get_stats()
    # foo remains unchanged while baz is changed
    assert file_true.read_log() == """foo,bar,baz\n"""


def test_literal_generator_assignment(config, syslog_ng):
    (file_true, file_false) = create_config(
        config, """
$MSG = json();
$MSG.foo = {"answer": 42, "leet": 1337};
$MSG["bar"] = {"answer+1": 43, "leet+1": 1338};
$MSG.list = ["will be replaced"];
$MSG.list[0] = [1, 2, 3];
$MSG.list[] = [4, 5, 6];
""",
    )
    syslog_ng.start(config)

    assert file_true.get_stats()["processed"] == 1
    assert "processed" not in file_false.get_stats()
    assert file_true.read_log() == """{"foo":{"answer":42,"leet":1337},"bar":{"answer+1":43,"leet+1":1338},"list":[[1,2,3],[4,5,6]]}\n"""


def test_literal_generator_casted_assignment(config, syslog_ng):
    (file_true, file_false) = create_config(
        config, """
$MSG = json();
$MSG.foo = json({"answer": 42, "leet": 1337});
$MSG["bar"] = json({"answer+1": 43, "leet+1": 1338});
$MSG.list = json_array(["will be replaced"]);
$MSG.list[0] = json_array([1, 2, 3]);
$MSG.list[] = json_array([4, 5, 6]);
""",
    )
    syslog_ng.start(config)

    assert file_true.get_stats()["processed"] == 1
    assert "processed" not in file_false.get_stats()
    assert file_true.read_log() == """{"foo":{"answer":42,"leet":1337},"bar":{"answer+1":43,"leet+1":1338},"list":[[1,2,3],[4,5,6]]}\n"""


def test_function_call(config, syslog_ng):
    (file_true, file_false) = create_config(
        config, """
$list = json_array();
$list[] = "foo";
$list[] = "bar";
$list[] = "baz";
$MSG = example_echo($list);
""",
    )
    syslog_ng.start(config)

    assert file_true.get_stats()["processed"] == 1
    assert "processed" not in file_false.get_stats()
    # foo remains unchanged while baz is changed
    assert file_true.read_log() == """foo,bar,baz\n"""


def test_ternary_operator_true(config, syslog_ng):
    (file_true, file_false) = create_config(
        config, """
    $MSG = true?${values.true_string}:${values.false_string};
""",
    )
    syslog_ng.start(config)

    assert file_true.get_stats()["processed"] == 1
    assert "processed" not in file_false.get_stats()
    assert file_true.read_log() == "boolean:true\n"


def test_ternary_operator_false(config, syslog_ng):
    (file_true, file_false) = create_config(
        config, """
    $MSG = false?${values.true_string}:${values.false_string};
""",
    )
    syslog_ng.start(config)

    assert file_true.get_stats()["processed"] == 1
    assert "processed" not in file_false.get_stats()
    assert file_true.read_log() == "boolean:false\n"


def test_ternary_operator_expression_true(config, syslog_ng):
    (file_true, file_false) = create_config(
        config, """
    $MSG = (0 === 0)?${values.true_string}:${values.false_string};
""",
    )
    syslog_ng.start(config)

    assert file_true.get_stats()["processed"] == 1
    assert "processed" not in file_false.get_stats()
    assert file_true.read_log() == "boolean:true\n"


def test_ternary_operator_expression_false(config, syslog_ng):
    (file_true, file_false) = create_config(
        config, """
    $MSG = (0 === 1)?${values.true_string}:${values.false_string};
""",
    )
    syslog_ng.start(config)

    assert file_true.get_stats()["processed"] == 1
    assert "processed" not in file_false.get_stats()
    assert file_true.read_log() == "boolean:false\n"


def test_ternary_operator_inline_ternary_expression_true(config, syslog_ng):
    (file_true, file_false) = create_config(
        config, """
    $MSG = (0 === 0)?("foo" eq "foo"? ${values.true_string} : "inner:false"):${values.false_string};
""",
    )
    syslog_ng.start(config)

    assert file_true.get_stats()["processed"] == 1
    assert "processed" not in file_false.get_stats()
    assert file_true.read_log() == "boolean:true\n"


def test_ternary_operator_inline_ternary_expression_false(config, syslog_ng):
    (file_true, file_false) = create_config(
        config, """
    $MSG = (0 === 0)?("foo" eq "bar"? ${values.true_string} : "inner:false"):${values.false_string};
""",
    )
    syslog_ng.start(config)

    assert file_true.get_stats()["processed"] == 1
    assert "processed" not in file_false.get_stats()
    assert file_true.read_log() == "inner:false\n"


def test_ternary_return_condition_expression_value_without_true_branch(config, syslog_ng):
    (file_true, file_false) = create_config(
        config, """
    $MSG = ${values.true_string}?:${values.false_string};
""",
    )

    syslog_ng.start(config)

    assert file_true.get_stats()["processed"] == 1
    assert "processed" not in file_false.get_stats()
    assert file_true.read_log() == "boolean:true\n"


def test_if_condition_without_else_branch_match(config, syslog_ng):
    (file_true, file_false) = create_config(
        config, """
    $out = "default";
    if (true) {
        $out = "matched";
    };
    $MSG = $out;
""",
    )
    syslog_ng.start(config)

    assert file_true.get_stats()["processed"] == 1
    assert "processed" not in file_false.get_stats()
    assert file_true.read_log() == "matched\n"


def test_if_condition_without_else_branch_nomatch(config, syslog_ng):
    (file_true, file_false) = create_config(
        config, """
    $out = "default";
    if (false) {
        $out = "matched";
    };
    $MSG = $out;
""",
    )
    syslog_ng.start(config)

    assert file_true.get_stats()["processed"] == 1
    assert "processed" not in file_false.get_stats()
    assert file_true.read_log() == "default\n"


def test_if_condition_no_matching_condition(config, syslog_ng):
    (file_true, file_false) = create_config(
        config, """
    $out = "default";
    if (false) {
        $out = "matched";
    } elif (false) {
        $out = "elif-matched";
    };
    $MSG = $out;
""",
    )
    syslog_ng.start(config)

    assert file_true.get_stats()["processed"] == 1
    assert "processed" not in file_false.get_stats()
    assert file_true.read_log() == "default\n"


def test_if_condition_matching_main_condition(config, syslog_ng):
    (file_true, file_false) = create_config(
        config, """
    $out = "default";
    if (true) {
        $out = "matched";
    } elif (true) {
        $out = "elif-matched";
    } else {
        $out = "else-matched";
    };
    $MSG = $out;
""",
    )
    syslog_ng.start(config)

    assert file_true.get_stats()["processed"] == 1
    assert "processed" not in file_false.get_stats()
    assert file_true.read_log() == "matched\n"


def test_if_condition_matching_elif_condition(config, syslog_ng):
    (file_true, file_false) = create_config(
        config, """
    $out = "default";
    if (false) {
        $out = "matched";
    } elif (true) {
        $out = "elif-matched";
    } else {
        $out = "else-matched";
    };
    $MSG = $out;
""",
    )
    syslog_ng.start(config)

    assert file_true.get_stats()["processed"] == 1
    assert "processed" not in file_false.get_stats()
    assert file_true.read_log() == "elif-matched\n"


def test_if_condition_matching_else_condition(config, syslog_ng):
    (file_true, file_false) = create_config(
        config, """
    $out = "default";
    if (false) {
        $out = "matched";
    } elif (false) {
        $out = "elif-matched";
    } else {
        $out = "else-matched";
    };
    $MSG = $out;
""",
    )
    syslog_ng.start(config)

    assert file_true.get_stats()["processed"] == 1
    assert "processed" not in file_false.get_stats()
    assert file_true.read_log() == "else-matched\n"


def test_if_condition_matching_expression(config, syslog_ng):
    (file_true, file_false) = create_config(
        config, """
    $out = "default";
    if ("foo" eq "foo") {
        $out = "matched";
    };
    $MSG = $out;
""",
    )
    syslog_ng.start(config)

    assert file_true.get_stats()["processed"] == 1
    assert "processed" not in file_false.get_stats()
    assert file_true.read_log() == "matched\n"


def test_if_condition_non_matching_expression(config, syslog_ng):
    (file_true, file_false) = create_config(
        config, """
    $out = "default";
    if ("foo" eq "bar") {
        $out = "matched";
    };
    $MSG = $out;
""",
    )
    syslog_ng.start(config)

    assert file_true.get_stats()["processed"] == 1
    assert "processed" not in file_false.get_stats()
    assert file_true.read_log() == "default\n"


def test_isset_existing_value_not_in_scope_yet(config, syslog_ng):
    (file_true, file_false) = create_config(
        config, """
    isset($MSG);
""",
    )
    syslog_ng.start(config)

    assert file_true.get_stats()["processed"] == 1
    assert "processed" not in file_false.get_stats()


def test_isset_existing_value_already_in_scope(config, syslog_ng):
    (file_true, file_false) = create_config(
        config, """
    $MSG;
    isset($MSG);
""",
    )
    syslog_ng.start(config)

    assert file_true.get_stats()["processed"] == 1
    assert "processed" not in file_false.get_stats()


def test_isset_inexisting_value_not_in_scope_yet(config, syslog_ng):
    (file_true, file_false) = create_config(
        config, """
    not isset($almafa);
""",
    )
    syslog_ng.start(config)

    assert file_true.get_stats()["processed"] == 1
    assert "processed" not in file_false.get_stats()


def test_isset_inexisting_value_already_in_scope(config, syslog_ng):
    (file_true, file_false) = create_config(
        config, """
    $almafa = "x";
    unset($almafa);
    not isset($almafa);
""",
    )
    syslog_ng.start(config)

    assert file_true.get_stats()["processed"] == 1
    assert "processed" not in file_false.get_stats()


def test_isset(config, syslog_ng):
    (file_true, file_false) = create_config(
        config, """
    $MSG = json();
    $MSG.inner_key = "foo";

    isset(${values.int});
    not isset($almafa);
    isset($MSG["inner_key"]);
    not isset($MSG["almafa"]);
""",
    )
    syslog_ng.start(config)

    assert file_true.get_stats()["processed"] == 1
    assert "processed" not in file_false.get_stats()


def test_unset_value_not_in_scope_yet(config, syslog_ng):
    (file_true, file_false) = create_config(
        config, """
    unset($MSG);
""",
    )
    syslog_ng.start(config)

    assert file_true.get_stats()["processed"] == 1
    assert "processed" not in file_false.get_stats()
    assert file_true.read_log() == "\n"


def test_unset_value_already_in_scope(config, syslog_ng):
    (file_true, file_false) = create_config(
        config, """
    $MSG;
    unset($MSG);
""",
    )
    syslog_ng.start(config)

    assert file_true.get_stats()["processed"] == 1
    assert "processed" not in file_false.get_stats()
    assert file_true.read_log() == "\n"


def test_unset_existing_key(config, syslog_ng):
    (file_true, file_false) = create_config(
        config, """
    $MSG = json();
    $MSG["foo"] = "bar";
    unset($MSG["foo"]);
    not isset($MSG["foo"]);
""",
    )
    syslog_ng.start(config)

    assert file_true.get_stats()["processed"] == 1
    assert "processed" not in file_false.get_stats()
    assert file_true.read_log() == "{}\n"


def test_unset_inexisting_key(config, syslog_ng):
    (file_true, file_false) = create_config(
        config, """
    $MSG = json();
    unset($MSG["foo"]);
    not isset($MSG["foo"]);
""",
    )
    syslog_ng.start(config)

    assert file_true.get_stats()["processed"] == 1
    assert "processed" not in file_false.get_stats()
    assert file_true.read_log() == "{}\n"


def test_setting_an_unset_key_will_contain_the_right_value(config, syslog_ng):
    (file_true, file_false) = create_config(
        config, """
    $MSG = json();
    $MSG["foo"] = "first";
    unset($MSG["foo"]);
    not isset($MSG["foo"]);
    $MSG["foo"] = "second";
""",
    )
    syslog_ng.start(config)

    assert file_true.get_stats()["processed"] == 1
    assert "processed" not in file_false.get_stats()
    assert file_true.read_log() == '{"foo":"second"}\n'


def test_unset(config, syslog_ng):
    (file_true, file_false) = create_config(
        config, """
    $MSG = json();
    $MSG["inner_key"] = "foo";
    $arr = json_array();
    $arr[] = "first";
    $arr[] = "second";

    unset(${values.int});
    unset($almafa);
    unset($MSG["inner_key"]);
    unset($MSG["almafa"]);
    unset($arr[0]);

    not isset(${values.int});
    not isset($almafa);
    not isset($MSG["inner_key"]);
    not isset($MSG["almafa"]);
    isset($arr[0]);
    not isset($arr[1]);
    unset($MSG);
""",
    )
    syslog_ng.start(config)

    assert file_true.get_stats()["processed"] == 1
    assert "processed" not in file_false.get_stats()
    assert file_true.read_log() == "\n"


def test_strptime_error_result(config, syslog_ng):
    _ = create_config(
        config, """
        $MSG = strptime("2024-04-10T08:09:10Z"); # wrong arg set
""",
    )
    with pytest.raises(Exception):
        syslog_ng.start(config)


def test_strptime_success_result(config, syslog_ng):
    (file_true, file_false) = create_config(
        config, """
        $MSG = strptime("2024-04-10T08:09:10Z", "%Y-%m-%dT%H:%M:%S%z");
""",
    )
    syslog_ng.start(config)

    assert file_true.get_stats()["processed"] == 1
    assert "processed" not in file_false.get_stats()
    assert file_true.read_log() == "1712736550.000000+00:00\n"


def test_strptime_failure_result(config, syslog_ng):
    (file_true, file_false) = create_config(
        config, """
        $MSG = string(strptime("2024-04-10T08:09:10Z", "no", "valid", "parse", "fmt"));
""",
    )
    syslog_ng.start(config)

    assert file_true.get_stats()["processed"] == 1
    assert "processed" not in file_false.get_stats()
    assert file_true.read_log() == "null\n"


def test_len(config, syslog_ng):
    (file_true, file_false) = create_config(
        config, r"""
    $dict = json();
    $list = json_array();
    len($dict) == 0;
    len($list) == 0;

    $dict.foo = "bar";
    $list[] = "foo";
    len($dict) == 1;
    len($list) == 1;

    len(${values.str}) == 6;

    $MSG = "success";
""",
    )
    syslog_ng.start(config)

    assert file_true.get_stats()["processed"] == 1
    assert "processed" not in file_false.get_stats()
    assert file_true.read_log() == "success\n"


def test_regexp_match(config, syslog_ng):
    (file_true, file_false) = create_config(
        config, r"""
    ${values.str} =~ /string/;
    ${values.str} =~ /.*/;
    not (${values.str} =~ /foobar/);
    "\"" =~ /"/;
    $MSG = "success";
""",
    )
    syslog_ng.start(config)

    assert file_true.get_stats()["processed"] == 1
    assert "processed" not in file_false.get_stats()
    assert file_true.read_log() == "success\n"


def test_regexp_match_error_in_pattern(config, syslog_ng):
    _ = create_config(
        config, r"""
    "foo" =~ /(/;
""",
    )
    with pytest.raises(Exception):
        syslog_ng.start(config)


def test_regexp_search(config, syslog_ng):
    (file_true, file_false) = create_config(
        config, r"""
    $MSG = json();
    $MSG.unnamed = regexp_search("foobarbaz", /(foo)(bar)(baz)/);
    $MSG.named = regexp_search("foobarbaz", /(?<first>foo)(?<second>bar)(?<third>baz)/);
    $MSG.mixed = regexp_search("foobarbaz", /(?<first>foo)(bar)(?<third>baz)/);
    $MSG.force_list = json_array(regexp_search("foobarbaz", /(?<first>foo)(bar)(?<third>baz)/));
    $MSG.force_dict = json(regexp_search("foobarbaz", /(foo)(bar)(baz)/));

    $MSG.no_match_unnamed = regexp_search("foobarbaz", /(almafa)/);
    if (len($MSG.no_match_unnamed) == 0) {
        $MSG.no_match_unnamed_handling = true;
    };

    $MSG.no_match_named = regexp_search("foobarbaz", /(?<first>almafa)/);
    if (len($MSG.no_match_named) == 0) {
        $MSG.no_match_named_handling = true;
    };
""",
    )
    syslog_ng.start(config)

    assert file_true.get_stats()["processed"] == 1
    assert "processed" not in file_false.get_stats()
    assert json.loads(file_true.read_log()) == {
        "unnamed": ["foobarbaz", "foo", "bar", "baz"],
        "named": {"0": "foobarbaz", "first": "foo", "second": "bar", "third": "baz"},
        "mixed": {"0": "foobarbaz", "first": "foo", "2": "bar", "third": "baz"},
        "force_list": ["foobarbaz", "foo", "bar", "baz"],
        "force_dict": {"0": "foobarbaz", "1": "foo", "2": "bar", "3": "baz"},
        "no_match_unnamed": [],
        "no_match_unnamed_handling": True,
        "no_match_named": {},
        "no_match_named_handling": True,
    }


def test_regexp_search_error_in_pattern(config, syslog_ng):
    _ = create_config(
        config, r"""
    $MSG = json(regexp_search("foo", /(/));
""",
    )
    with pytest.raises(Exception):
        syslog_ng.start(config)


def test_parse_kv_default_option_set_is_skippable(config, syslog_ng):
    (file_true, file_false) = create_config(
        config, """
    $MSG = parse_kv("foo=bar, thisisstray bar=baz");
    """,
    )
    syslog_ng.start(config)

    assert file_true.get_stats()["processed"] == 1
    assert "processed" not in file_false.get_stats()
    assert file_true.read_log() == '{"foo":"bar","bar":"baz"}\n'


def test_parse_kv_value_separator(config, syslog_ng):
    (file_true, file_false) = create_config(
        config, """
    $MSG = parse_kv("foo@bar, bar@baz", value_separator="@");
    """,
    )
    syslog_ng.start(config)

    assert file_true.get_stats()["processed"] == 1
    assert "processed" not in file_false.get_stats()
    assert file_true.read_log() == "{\"foo\":\"bar\",\"bar\":\"baz\"}\n"


def test_parse_kv_value_separator_use_first_character(config, syslog_ng):
    (file_true, file_false) = create_config(
        config, """
    $MSG = parse_kv("foo@bar, bar@baz", value_separator="@!$");
    """,
    )
    syslog_ng.start(config)

    assert file_true.get_stats()["processed"] == 1
    assert "processed" not in file_false.get_stats()
    assert file_true.read_log() == "{\"foo\":\"bar\",\"bar\":\"baz\"}\n"


def test_parse_kv_pair_separator(config, syslog_ng):
    (file_true, file_false) = create_config(
        config, """
    $MSG = parse_kv("foo=bar#bar=baz", pair_separator="#");
    """,
    )
    syslog_ng.start(config)

    assert file_true.get_stats()["processed"] == 1
    assert "processed" not in file_false.get_stats()
    assert file_true.read_log() == "{\"foo\":\"bar\",\"bar\":\"baz\"}\n"


def test_parse_kv_stray_words_value_name(config, syslog_ng):
    (file_true, file_false) = create_config(
        config, """
    $MSG = parse_kv("foo=bar, thisisstray bar=baz", stray_words_key="stray_words");
    """,
    )
    syslog_ng.start(config)

    assert file_true.get_stats()["processed"] == 1
    assert "processed" not in file_false.get_stats()
    assert file_true.read_log() == "{\"foo\":\"bar\",\"bar\":\"baz\",\"stray_words\":\"thisisstray\"}\n"


def test_slash_string_features(config, syslog_ng):
    cfg = r"""
            $MSG = json();
            $MSG.base = /foo bar/;
            $MSG.line_break = /foo
bar/;
            $MSG.joint_lines = /foo \
bar/;
            ## escaped characters
            $MSG.escaped_backslash = /foo\\bar/;
            $MSG.escaped_slash = /foo\/bar/;
            $MSG.non_escaped_single_quotes = /foo'bar/;
            $MSG.non_escaped_double_quotes = /foo"bar/;

            ## special characters
            $MSG.new_line = /foo\nbar/;
            $MSG.tab = /foo\tbar/;
            $MSG.carrige_return = /foo\rbar/;
            $MSG.vertical_tab = /foo\vbar/;
            $MSG.form_feed = /foo\fbar/;
            $MSG.backspace = /foo\bbar/;
            $MSG.alert = /foo\abar/;

            ##
            $MSG.hexadecimal_value = /foo\x40bar/;
            $MSG.octal_value = /foo\o100bar/;
        """

    (file_true, file_false) = create_config(
        config, cfg,
    )
    syslog_ng.start(config)

    assert file_true.get_stats()["processed"] == 1
    assert "processed" not in file_false.get_stats()
    exp = (
        r"""{"base":"foo bar","""
        r""""line_break":"foo\nbar","""
        r""""joint_lines":"foo bar","""
        r""""escaped_backslash":"foo\\bar","""
        r""""escaped_slash":"foo\/bar","""
        r""""non_escaped_single_quotes":"foo'bar","""
        r""""non_escaped_double_quotes":"foo\"bar","""
        r""""new_line":"foo\nbar","""
        r""""tab":"foo\tbar","""
        r""""carrige_return":"foo\rbar","""
        r""""vertical_tab":"foo\u000bbar","""
        r""""form_feed":"foo\fbar","""
        r""""backspace":"foo\bbar","""
        r""""alert":"foo\u0007bar","""
        r""""hexadecimal_value":"foo@bar","""
        r""""octal_value":"foo@bar"}""" + "\n"
    )
    res = file_true.read_log()
    assert res == exp



#############################################################################
# Copyright (c) 2015-2016 Balabit
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 2 as published
# by the Free Software Foundation, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#
# As an additional exemption you are allowed to compile & link against the
# OpenSSL libraries as published by the OpenSSL project. See the file
# COPYING for details.
#
#############################################################################

from .test_completerlang import CompleterLangTestCase
from ..tflang import TemplateFunctionLang


class TestTFLang(CompleterLangTestCase):
    def setUp(self):
        self._parser = TemplateFunctionLang()

    def test_template_func_is_a_command_and_a_sequence_of_arguments(self):
        self._assert_token_follows("", ["COMMAND"])
        self._assert_token_follows("foo ", ["ARG"])
        self._assert_token_follows("foo bar ", ["ARG"])
        self._assert_token_follows("foo bar baz", ["ARG"])

    def test_format_json_arguments(self):
        self._assert_token_follows("format-json", ["COMMAND_FORMAT_JSON"])
        self._assert_token_follows("format-json ", ["ARG", "OPT__KEY", "OPT__PAIR"])
        self._assert_token_follows("format-json --key ", ["name_value_name"])
        self._assert_token_follows("format-json --pair ", ["name_value_pair"])
        self._assert_token_follows("format-json --scope ", ["value_pairs_scope"])
        self._assert_token_follows("format-json --key * --pair a=$b --scope ", ["value_pairs_scope"])

    def test_echo_arguments(self):
        self._assert_token_follows("echo ", ["template", "ARG"])
        self._assert_token_follows("echo $MSG ", ["template", "ARG"])
        self._assert_token_follows("echo $MSG $MSG ", ["template", "ARG"])



#!/usr/bin/env python3
#############################################################################
# Copyright (c) 2020 Balabit
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 2 as published
# by the Free Software Foundation, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#
# As an additional exemption you are allowed to compile & link against the
# OpenSSL libraries as published by the OpenSSL project. See the file
# COPYING for details.
#
#############################################################################

import re
import sys
import textwrap
from argparse import ArgumentParser
from pathlib import Path
from subprocess import PIPE, Popen

news_dir = Path(__file__).resolve().parent
root_dir = news_dir.parent
newsfile = root_dir / 'NEWS.md'

exclude_contributor_list = [
    "github-actions",
    "dependabot[bot]",
]


blocks = [
    ('Features', 'feature-*.md'),
    ('Bugfixes', 'bugfix-*.md'),
    ('Packaging', 'packaging-*.md'),
    ('Notes to developers', 'developer-note-*.md'),
    ('Other changes', 'other-*.md'),
]


def print_usage_if_needed():
    ArgumentParser(usage="\rCreates NEWS.md file from the entries in the news/ folder.\n"
                         "It also deletes the entry files.").parse_args()


def _exec(command):
    proc = Popen(command, cwd=str(root_dir), stdout=PIPE, shell=True)
    stdout, _ = proc.communicate()
    stdout = stdout.decode()
    return stdout


def create_block(block_name, files):
    block = '## {}\n\n'.format(block_name)
    for f in files:
        entry = ''
        match_pr_id = re.search(r'(\d+)(-\d+)?.md$', f.name)
        if not match_pr_id:
            sys.exit('Invalid filename: {}'.format(f.name))

        pr_id = match_pr_id.group(1)

        entry += '  * {}\n([#{}](https://github.com/syslog-ng/syslog-ng/pull/{}))'.format(f.read_text().rstrip(), pr_id, pr_id)
        entry = entry.replace('\n', '\n    ')
        entry = entry.replace('\n    \n', '\n\n')
        block += entry + '\n\n'
    block += '\n'
    return block


def get_last_version():
    stdout = _exec(r'git show HEAD:NEWS.md')
    return stdout[:stdout.index('\n')]


def get_next_version():
    next_version = (root_dir / 'VERSION.txt').read_text().rstrip()
    return next_version

def create_version():
    next_version = get_next_version()
    return '{}\n{}\n\n'.format(next_version, len(next_version) * '=')

def create_highlights_block():
    return '## Highlights\n' \
           '\n' \
           '<Fill this block manually from the blocks below>\n' \
           '\n'


def create_standard_blocks():
    standard_blocks = ''
    for block_name, glob in blocks:
        entries = list(news_dir.glob(glob))
        if len(entries) > 0:
            standard_blocks += create_block(block_name, entries)
    return standard_blocks

def check_if_news_entries_are_present():
    return any(news_dir.glob("*-*.md"))

def create_credits_block():
    def wrap(contributors):
        concated = ", ".join([c.replace(" ", "\0") for c in contributors])
        return textwrap.fill(concated, width=70).replace("\0", " ")

    stdout = _exec(r'git rev-list --no-merges --format=format:%aN syslog-ng-' + get_last_version() + r'..HEAD | '
                   r'grep -Ev "^commit [a-z0-9]{40}$" | sort | uniq')
    contributors = stdout.rstrip().split('\n')
    contributors = filter(lambda x : x not in exclude_contributor_list, contributors)
    contributors = sorted(set(contributors))

    return '## Credits\n' \
           '\n' \
           'syslog-ng is developed as a community project, and as such it relies\n' \
           'on volunteers, to do the work necessarily to produce syslog-ng.\n' \
           '\n' \
           'Reporting bugs, testing changes, writing code or simply providing\n' \
           'feedback are all important contributions, so please if you are a user\n' \
           'of syslog-ng, contribute.\n' \
           '\n' \
           'We would like to thank the following people for their contribution:\n' \
           '\n' \
           '{}\n'.format(wrap(contributors))


def create_newsfile(news):
    newsfile.write_text(news)
    print('Newsfile created at {}\n'.format(newsfile.resolve()))


def cleanup():
    print("Cleaning up entry files with `git rm news/*-*.md`:")
    _exec("git rm news/*-*.md")


def create_news_content():
    news = create_version()
    news += create_highlights_block()
    news += create_standard_blocks()
    news += create_credits_block()
    return news

def check_if_news_is_already_uptodate():
    return get_last_version() == get_next_version()

def main():
    print_usage_if_needed()

    if check_if_news_is_already_uptodate():
        if check_if_news_entries_are_present():
            print('NEWS.md file is already up-to-date with VERSION.txt but news entries still exist (news/*.md).\n'
                  'Remove NEWS entries or bump the VERSION.txt file.\n')
            return 1
        print("NEWS file is already up-to-date, no new NEWS entries, assuming it has been manually prepared")
    else:
        news = create_news_content()
        create_newsfile(news)
        cleanup()
    return 0


if __name__ == '__main__':
    sys.exit(main())



#!/usr/bin/env python
#############################################################################
# Copyright (c) 2024 Balazs Scheidler <balazs.scheidler@axoflow.com>
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 2 as published
# by the Free Software Foundation, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#
# As an additional exemption you are allowed to compile & link against the
# OpenSSL libraries as published by the OpenSSL project. See the file
# COPYING for details.
#
#############################################################################
from src.syslog_ng_ctl.prometheus_stats_handler import MetricFilter


def test_metrics_probe(config, syslog_ng):
    config.update_global_options(stats_level=1)
    generator_source = config.create_example_msg_generator_source(num=1, template=config.stringify("message"), values="dim1 => value1 dim2 => value2")

    probe1 = config.create_metrics_probe(key="test_metric1", labels=config.arrowed_options({"label1": config.stringify("$dim1")}))
    probe2 = config.create_metrics_probe(key="test_metric2", labels=config.arrowed_options({"label2": config.stringify("$dim2")}))

    file_destination = config.create_file_destination(file_name="output.log", template=config.stringify("$MSG\n"))
    config.create_logpath(statements=[generator_source, probe1, probe2, file_destination])

    syslog_ng.start(config)

    # message goes through
    assert file_destination.read_log().strip() == "message"

    samples = config.get_prometheus_samples([MetricFilter("syslogng_test_metric1", {})])
    assert len(samples) > 0
    assert samples[0].labels == {'label1': 'value1'}

    samples = config.get_prometheus_samples([MetricFilter("syslogng_test_metric2", {})])
    assert len(samples) > 0
    assert samples[0].labels == {'label2': 'value2'}



#############################################################################
# Copyright (c) 2022 Balazs Scheidler <bazsi77@gmail.com>
# Copyright (c) 2023 Attila Szakacs
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 2 as published
# by the Free Software Foundation, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#
# As an additional exemption you are allowed to compile & link against the
# OpenSSL libraries as published by the OpenSSL project. See the file
# COPYING for details.
#
#############################################################################
from syslogng.modules import hypr
import base64


def test_config_generator_all_options(mocker):
    mocker.patch("requests.get", return_value=mocker.Mock(**{
        'status_code': 200,
        'json': mocker.Mock(return_value=[
            {"appID": "HYPRDefaultApplication"},
            {"appID": "HYPRDefaultWorkstationApplication"},
            {"appID": "rp_foo"},
        ])
    }))

    bearer_token = b'bearer_token'
    base64_encoded_bearer_token = base64.b64encode(bearer_token).decode('utf8')
    hypr_args = {
        'url': '"https://dummy.hypr.com/"',
        'bearer_token': '"' + base64_encoded_bearer_token + '"',
    }

    snippet = hypr._hypr_config_generator(hypr_args)
    expected_snippet = """
        python-fetcher(
            class("syslogng.modules.hypr.HyprAuditSource")
            options(
                "url" => "https://dummy.hypr.com/"
                "rp_app_id" => "rp_foo"
                "bearer_token" => "%s"
                "page_size" => 100
                "initial_hours" => 4
                "ignore_persistence" => False
                "log_level" => "INFO"
            )
            flags()
            persist-name("hypr-https://dummy.hypr.com/-rp_foo")
            fetch-no-data-delay(60)
        );""" % base64_encoded_bearer_token

    assert expected_snippet in snippet


def test_config_generator_all_options(mocker):
    mocker.patch("requests.get", return_value=mocker.Mock(**{
        'status_code': 200,
        'json': mocker.Mock(return_value=[{"appID": "rp_foo"}, {"appID": "rp_bar"}, {"appID": "rp_baz"}])
    }))

    bearer_token = b'bearer_token'
    base64_encoded_bearer_token = base64.b64encode(bearer_token).decode('utf8')
    hypr_args = {
        'url': '"https://dummy.hypr.com/"',
        'bearer_token': '"' + base64_encoded_bearer_token + '"',
        'page_size': "123",
        'initial_hours': "9",
        'log_level': 'ERROR',
        'application_skiplist': '"rp_baz", "rp_app"',
        'ignore_persistence': "yes",
        'persist_name': '"custom-persist-name"',
        'fetch_no_data_delay': "42",
        'flags': 'no-parse',
    }

    snippet = hypr._hypr_config_generator(hypr_args)
    expected_snippets = [
        """
        python-fetcher(
            class("syslogng.modules.hypr.HyprAuditSource")
            options(
                "url" => "https://dummy.hypr.com/"
                "rp_app_id" => "rp_foo"
                "bearer_token" => "%s"
                "page_size" => 123
                "initial_hours" => 9
                "ignore_persistence" => yes
                "log_level" => "ERROR"
            )
            flags(no-parse)
            persist-name("custom-persist-name-rp_foo")
            fetch-no-data-delay(42)
        );""" % base64_encoded_bearer_token,
        """
        python-fetcher(
            class("syslogng.modules.hypr.HyprAuditSource")
            options(
                "url" => "https://dummy.hypr.com/"
                "rp_app_id" => "rp_bar"
                "bearer_token" => "%s"
                "page_size" => 123
                "initial_hours" => 9
                "ignore_persistence" => yes
                "log_level" => "ERROR"
            )
            flags(no-parse)
            persist-name("custom-persist-name-rp_bar")
            fetch-no-data-delay(42)
        );""" % base64_encoded_bearer_token,
    ]

    for expected_snippet in expected_snippets:
        assert expected_snippet in snippet



#############################################################################
# Copyright (c) 2007-2010 Balabit
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 2 as published
# by the Free Software Foundation, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#
# As an additional exemption you are allowed to compile & link against the
# OpenSSL libraries as published by the OpenSSL project. See the file
# COPYING for details.
#
#############################################################################

import struct, stat, re
from socket import *
import ssl
import os, sys, errno

from log import *

syslog_prefix = "2004-09-07T10:43:21+01:00 bzorp prog[12345]:"
syslog_new_prefix = "2004-09-07T10:43:21+01:00 bzorp prog 12345 - -"
session_counter = 0
need_to_flush = False
padding = 'x' * 250

class MessageSender(object):
    def __init__(self, repeat=100, new_protocol=0, dgram=0):
        self.repeat = repeat
        self.new_protocol = new_protocol
        self.dgram = dgram

    def sendMessages(self, msg, pri=7):
        global session_counter
        global need_to_flush
        global syslog_prefix
        global syslog_prefix_new

        need_to_flush = True


        print_user("generating %d messages using transport %s" % (self.repeat, str(self)))

        self.initSender()
        expected = []

        for counter in range(1, self.repeat):
            if self.new_protocol == 0:
                line = '<%d>%s %s %03d/%05d %s %s' % (pri, syslog_prefix, msg, session_counter, counter, str(self), padding)
            else:
                line = '<%d>1 %s %s %03d/%05d %s %s' % (pri, syslog_new_prefix, msg, session_counter, counter, str(self), padding)

            # add framing on tcp with new protocol
            if self.dgram == 0 and self.new_protocol == 1:
                line = '%d %s' % (len(line), line)
            self.sendMessage(line) # file or socket
        expected.append((msg, session_counter, self.repeat))
        session_counter = session_counter + 1
        return expected

class SocketSender(MessageSender):
    def __init__(self, family, sock_name, dgram=0, send_by_bytes=0, terminate_seq='\n', repeat=100, ssl=0, new_protocol=0):
        MessageSender.__init__(self, repeat, new_protocol, dgram)
        self.family = family
        self.sock_name = sock_name
        self.sock = None
        self.dgram = dgram
        self.send_by_bytes = send_by_bytes
        self.terminate_seq = terminate_seq
        self.ssl = ssl
        self.new_protocol = new_protocol
    def initSender(self):
        if self.dgram:
            self.sock = socket(self.family, SOCK_DGRAM)
        else:
            self.sock = socket(self.family, SOCK_STREAM)

        self.sock.connect(self.sock_name)
        if self.dgram:
                self.sock.send(''.encode())
        if sys.platform == 'linux2':
                self.sock.setsockopt(SOL_SOCKET, SO_SNDTIMEO, struct.pack('ll', 3, 0))
        if not self.dgram and self.ssl:
                self.sock = ssl.SSLContext(ssl.PROTOCOL_TLSv1_2).wrap_socket(self.sock)


    def sendMessage(self, msg):
        line = '%s%s' % (msg, self.terminate_seq)
        if self.send_by_bytes:
            for c in line:
                try:
                    if self.ssl:
                        self.sock.write(c.encode())
                    else:
                        self.sock.send(c.encode())
                except error as e:
                    if e[0] == errno.ENOBUFS:
                        print_user('got ENOBUFS, sleeping...')
                        time.sleep(0.5)
                        repeat = True
                    else:
                        print_user("hmm... got an error to the 'send' call, maybe syslog-ng is not accepting messages?")
                        raise
        else:
            repeat = True
            while repeat:
                try:
                    repeat = False

                    # WTF? SSLObject only has write, whereas sockets only have send methods
                    if self.ssl:
                        self.sock.write(line.encode())
                    else:
                        self.sock.send(line.encode())

                    if self.dgram:
                        time.sleep(0.01)
                except error as e:
                    if e[0] == errno.ENOBUFS:
                        print_user('got ENOBUFS, sleeping...')
                        time.sleep(0.5)
                        repeat = True
                    else:
                        print_user("hmm... got an error to the 'send' call, maybe syslog-ng is not accepting messages?")
                        raise

    def __str__(self):
        if self.family == AF_UNIX:
            if self.dgram:
                return 'unix-dgram(%s)' % (self.sock_name)
            else:
                return 'unix-stream(%s)' % (self.sock_name)
        else:
            if self.dgram:
                return 'udp(%s)' % (self.sock_name,)
            elif not self.ssl:
                return 'tcp(%s)' % (self.sock_name,)
            else:
                return 'tls(%s)' % (self.sock_name,)


class FileSender(MessageSender):
    def __init__(self, file_name, padding=0, send_by_bytes=0, terminate_seq='\n', repeat=100):
        MessageSender.__init__(self, repeat)
        self.file_name = file_name
        self.padding = padding
        self.send_by_bytes = send_by_bytes
        self.terminate_seq = terminate_seq
        self.fd = None

        try:
            if stat.S_ISFIFO(os.stat(file_name).st_mode):
                self.is_pipe = True
            else:
                self.is_pipe = False
        except OSError:
            self.is_pipe = False

    def __del__(self):
        if self.fd:
            self.fd.flush()
            self.fd.close()

    def initSender(self):
        directory = os.path.dirname(self.file_name)
        if not os.path.exists(directory) and len(directory) > 0:
            os.makedirs(directory)
        if self.is_pipe:
            self.fd = open(self.file_name, "w")
        else:
            self.fd = open(self.file_name, "a")

    def sendMessages(self, msg):
        res = super(FileSender, self).sendMessages(msg)

        return res

    def sendMessage(self, msg):
        line = '%s%s' % (msg, self.terminate_seq)
        if self.padding:
            line += '\0' * (self.padding - len(line))
        if self.send_by_bytes:
            for c in line:
                self.fd.write(c)
                self.fd.flush()
        else:
            self.fd.write(line)
            self.fd.flush()

    def __str__(self):
        if self.is_pipe:
            if self.padding:
                return 'pipe(%s[%d])' % (self.file_name, self.padding)
            else:
                return 'pipe(%s)' % (self.file_name,)
        else:
            return 'file(%s)' % (self.file_name,)




#############################################################################
# Copyright (c) 2022 Balazs Scheidler <bazsi77@gmail.com>
# Copyright (c) 2023 Attila Szakacs
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 2 as published
# by the Free Software Foundation, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#
# As an additional exemption you are allowed to compile & link against the
# OpenSSL libraries as published by the OpenSSL project. See the file
# COPYING for details.
#
#############################################################################
from syslogng.modules import hypr
import base64
import pytest
import json


@pytest.fixture
def empty_config():
    return {}


@pytest.fixture
def minimal_config():
    bearer_token = b'bearer_token'
    base64_encoded_bearer_token = base64.b64encode(bearer_token).decode('utf8')
    return {
        'url': 'https://dummy.hypr.com',
        'bearer_token': base64_encoded_bearer_token,
        'rp_app_id': 'rpFoo'
    }


def test_hypr_audit_source_can_be_instantiated():

    sut = hypr.HyprAuditSource()
    assert sut is not None


def test_hypr_audit_source_with_invalid_configuration(caplog, empty_config):
    sut = hypr.HyprAuditSource()

    assert sut.init(empty_config) is False
    assert "Missing rp_app_id" in caplog.text


def test_hypr_audit_source_lifecycle_without_actually_fetching_messages(minimal_config):

    sut = hypr.HyprAuditSource()

    assert sut.init(minimal_config) is True
    assert sut.open() is True
    assert sut.close() is None
    sut.deinit()


def test_hypr_audit_source_complete_lifecycle(minimal_config, mocker):

    response_json = [
        {
            'metadata': {
                'totalRecords': 2,
                'totalPages': 1,
                'currentPage': 1,
            },
            'data': [
                {
                    'rpAppId': 'rp_foo',
                    'eventTimeInUTC': 1666427394,
                    'message': 'whatever first',
                    'counter': 0,
                },
                {
                    'rpAppId': 'rp_foo',
                    'eventTimeInUTC': 1666427394,
                    'message': 'whatever second',
                    'counter': 1,
                    'single_nested': {
                        'double_nested': {
                            'none': None,
                        },
                    },
                    'float': 123.456,
                    'string_list': ['foo', 'bar'],
                    'non_string_list': [1, 2],
                    'bool': True,
                },
            ]
        },
        {
            'metadata': {
                'totalRecords': 0,
                'totalPages': 1,
                'currentPage': 1,
            },
            'data': [],
        }
    ]

    m = mocker.Mock(side_effect=response_json)

    mocker.patch('requests.get', return_value=mocker.Mock(**{
        'status_code': 200,
        'json': mocker.Mock(side_effect=response_json)
    }))
    sut = hypr.HyprAuditSource()

    assert sut.init(minimal_config) is True
    assert sut.open() is True

    index = 0
    status, msg = sut.fetch()

    while status == sut.SUCCESS:
        if index == 0:
            assert msg['MESSAGE'] == b'whatever first'
            assert msg['PROGRAM'] == b'rp_foo'
            assert msg['.hypr.rpAppId'] == b'rp_foo'
            assert msg['.hypr.eventTimeInUTC'] == 1666427394
            assert msg['.hypr.message'] == b'whatever first'
            assert msg['.hypr.counter'] == 0
        elif index == 1:
            assert msg['MESSAGE'] == b'whatever second'
            assert msg['PROGRAM'] == b'rp_foo'
            assert msg['.hypr.rpAppId'] == b'rp_foo'
            assert msg['.hypr.eventTimeInUTC'] == 1666427394
            assert msg['.hypr.message'] == b'whatever second'
            assert msg['.hypr.counter'] == 1
            assert msg['.hypr.single_nested.double_nested.none'] is None
            assert msg['.hypr.float'] == 123.456
            assert msg['.hypr.string_list'] == ['foo', 'bar']
            assert msg['.hypr.non_string_list'] == b'[1, 2]'
            assert msg['.hypr.bool'] is True

        index += 1
        status, msg = sut.fetch()

    assert status == sut.NO_DATA
    assert index == 2

    assert sut.close() is None
    sut.deinit()


def test_hypr_audit_source_complete_lifecycle_no_parse(minimal_config, mocker):

    response_json = [
        {
            'metadata': {
                'totalRecords': 1,
                'totalPages': 1,
                'currentPage': 1,
            },
            'data': [
                {
                    'rpAppId': 'rp_foo',
                    'eventTimeInUTC': 1666427394,
                    'message': 'whatever first',
                    'counter': 1,
                    'single_nested': {
                        'double_nested': {
                            'none': None,
                        },
                    },
                    'float': 123.456,
                    'string_list': ['foo', 'bar'],
                    'non_string_list': [1, 2],
                    'bool': True,
                },
            ],
        }
    ]

    m = mocker.Mock(side_effect=response_json)

    mocker.patch('requests.get', return_value=mocker.Mock(**{
        'status_code': 200,
        'json': mocker.Mock(side_effect=response_json)
    }))
    sut = hypr.HyprAuditSource()
    sut.flags["parse"] = False

    assert sut.init(minimal_config) is True
    assert sut.open() is True

    status, msg = sut.fetch()

    assert status == sut.SUCCESS
    assert msg['MESSAGE'] == b'{"rpAppId": "rp_foo", "eventTimeInUTC": 1666427394, "message": "whatever first", "counter": 1, "single_nested": {"double_nested": {"none": null}}, "float": 123.456, "string_list": ["foo", "bar"], "non_string_list": [1, 2], "bool": true}'
    assert msg['PROGRAM'] == b'Hypr-rpFoo'

    assert sut.close() is None
    sut.deinit()



#############################################################################
# Copyright (c) 2007-2015 Balabit
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 2 as published
# by the Free Software Foundation, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#
# As an additional exemption you are allowed to compile & link against the
# OpenSSL libraries as published by the OpenSSL project. See the file
# COPYING for details.
#
#############################################################################

from globals import *
from log import *
from messagegen import *
from messagecheck import *
from control import *

config_poll = """@version: %(syslog_ng_version)s

options { ts_format(iso); chain_hostnames(no); keep_hostname(yes); threaded(yes); };

source s_wildcard { wildcard_file(
                    base_dir("wildcard")
                    filename_pattern("*.log")
                    monitor-method("poll")
                    recursive(yes));
                  };

destination d_wildcard { file("test-wildcard.log"); };

log { source(s_wildcard); destination(d_wildcard); };

""" % locals()

config_auto = """@version: %(syslog_ng_version)s

options { ts_format(iso); chain_hostnames(no); keep_hostname(yes); threaded(yes); };

source s_wildcard { wildcard_file(
                    base_dir("wildcard")
                    filename_pattern("*.log")
                    monitor-method("auto")
                    recursive(yes));
                  };

destination d_wildcard { file("test-wildcard.log"); };

log { source(s_wildcard); destination(d_wildcard); };

""" % locals()

config = {"poll": config_poll, "auto": config_auto}

settle_time=4

def create_source_files(recursive=False):
    messages = (
      'wildcard0',
      'wildcard1',
      'wildcard2',
      'wildcard3',
      'wildcard4',
      'wildcard5',
      'wildcard6',
      'wildcard7',
    )
    expected = []

    for ndx in range(0, len(messages)):
        if not recursive:
            s = FileSender('wildcard/%d.log' % (ndx % 4), repeat=100)
        else:
            s = FileSender('wildcard/wildcard%d/%d.log' % ((ndx % 2),(ndx % 4)), repeat=100)
        expected.extend(s.sendMessages(messages[ndx]))
    return expected

def test_wildcard_files():
    expected = create_source_files()

    if not check_file_expected('test-wildcard', expected, settle_time=settle_time):
        return False
    return True

def test_wildcard_recursion():
    expected = create_source_files(True)
    if not check_file_expected('test-wildcard', expected, settle_time=settle_time):
        return False
    return True

def file_get_contents(filename):
    with open(filename) as f:
        return f.read()

def test_wildcard_no_directory_exists():
    stop_syslogng()
    os.system("rm -rf wildcard")
    
    start_syslogng(file_get_contents("test.conf"))
    os.system("mkdir wildcard")

    expected = create_source_files()

    if not check_file_expected('test-wildcard', expected, settle_time=settle_time):
        return False
    return True

def test_wildcard_runtime_detection():
    expected = create_source_files()

    messagegen.need_to_flush = False
    print_user("waiting for syslog-ng to process files (%d sec)" % 5)
    time.sleep(5)
    if not check_file_expected('test-wildcard', expected, settle_time=settle_time):
        return False
    return True



#!/usr/bin/env python
#############################################################################
# Copyright (c) 2023 Balazs Scheidler <bazsi77@gmail.com>
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 2 as published
# by the Free Software Foundation, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#
# As an additional exemption you are allowed to compile & link against the
# OpenSSL libraries as published by the OpenSSL project. See the file
# COPYING for details.
#
#############################################################################


def test_sdata_parser(config, syslog_ng):
    config.update_global_options(stats_level=1)
    generator_source = config.create_example_msg_generator_source(num=1, template=config.stringify("[Originator@6876 sub=Vimsvc.ha-eventmgr opID=esxui-13c6-6b16 sid=5214bde6 user=root]"))
    sdata_parser = config.create_sdata_parser(prefix=config.stringify(".SDATA."))

    file_destination = config.create_file_destination(file_name="output.log", template=config.stringify("$SDATA\n"))
    config.create_logpath(statements=[generator_source, sdata_parser, file_destination])

    syslog_ng.start(config)
    assert file_destination.read_log().strip() == '[Originator@6876 sub="Vimsvc.ha-eventmgr" opID="esxui-13c6-6b16" sid="5214bde6" user="root"]'
    assert sdata_parser.get_query().get('discarded', -1) == 0



#!/usr/bin/env python
#############################################################################
# Copyright (c) 2015-2018 Balabit
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 2 as published
# by the Free Software Foundation, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#
# As an additional exemption you are allowed to compile & link against the
# OpenSSL libraries as published by the OpenSSL project. See the file
# COPYING for details.
#
#############################################################################
from src.syslog_ng_ctl.syslog_ng_ctl_executor import SyslogNgCtlExecutor


class SyslogNgCtlCli(object):
    def __init__(self, instance_paths):
        self.__syslog_ng_ctl_executor = SyslogNgCtlExecutor(instance_paths)

    def reload(self):
        return self.__syslog_ng_ctl_executor.run_command(command_short_name="reload", command=["reload"])

    def stop(self):
        return self.__syslog_ng_ctl_executor.run_command(command_short_name="stop", command=["stop"])

    def stats(self, reset):
        ctl_stats_command = self.__syslog_ng_ctl_executor.construct_ctl_stats_command(reset=reset)
        return self.__syslog_ng_ctl_executor.run_command(command_short_name="stats", command=ctl_stats_command)

    def stats_prometheus(self):
        ctl_stats_prometheus_command = self.__syslog_ng_ctl_executor.construct_ctl_stats_prometheus_command()
        return self.__syslog_ng_ctl_executor.run_command(command_short_name="stats_prometheus", command=ctl_stats_prometheus_command)

    def query(self, pattern, query_type):
        ctl_query_command = self.__syslog_ng_ctl_executor.construct_ctl_query_command(pattern, query_type)
        return self.__syslog_ng_ctl_executor.run_command(command_short_name="query", command=ctl_query_command)

    def credentials_add(self, credential, secret):
        ctl_credentials_command = self.__syslog_ng_ctl_executor.construct_ctl_credentials_command(credential, secret)
        return self.__syslog_ng_ctl_executor.run_command(command_short_name="credentials", command=ctl_credentials_command)

    def is_control_socket_alive(self):
        return self.stats(reset=False)["exit_code"] == 0



#!/usr/bin/env python
#############################################################################
# Copyright (c) 2023 Balazs Scheidler <bazsi77@gmail.com>
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 2 as published
# by the Free Software Foundation, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#
# As an additional exemption you are allowed to compile & link against the
# OpenSSL libraries as published by the OpenSSL project. See the file
# COPYING for details.
#
#############################################################################
from src.syslog_ng_config.renderer import render_statement


def create_config(config, test_config, msg="foobar"):
    file_true = config.create_file_destination(file_name="dest-true.log", template="'$MSG\n'")
    file_false = config.create_file_destination(file_name="dest-false.log", template="'$MSG\n'")
    file_after = config.create_file_destination(file_name="dest-after.log", template="'$MSG\n'")
    file_fallback = config.create_file_destination(file_name="dest-fallback.log", template="'$MSG\n'")
    file_dropall = config.create_file_destination(file_name="dest-dropall.log", template="'$MSG\n'")

    preamble = f"""
@version: {config.get_version()}

options {{ stats-level(1); }};

block filter true() {{
    "1" eq "1"
}};

block filter false() {{
    "0" eq "1"
}};

source genmsg {{
    example-msg-generator(num(1) template("{msg}"));
}};

destination dest_after {{
    {render_statement(file_after)};
}};

destination dest_true {{
    {render_statement(file_true)};
}};

destination dest_false {{
    {render_statement(file_false)};
}};

destination dest_fallback {{
    {render_statement(file_fallback)};
}};

destination dest_dropall {{
    channel {{
        filter {{ false(); }};
        destination {{ {render_statement(file_dropall)}; }};
    }};
}};

log {{
    source(genmsg);
    destination(dest_fallback);
    flags(fallback);
}};

"""
    config.set_raw_config(preamble + test_config)
    return (file_true, file_false, file_after, file_fallback, file_dropall)


def test_midpoint_destination_that_drops_all_messages_does_not_cause_message_to_be_unmatched(config, syslog_ng):
    # Rationale:
    #
    #   midpoint destinations which drop messages completely do it in their
    #   own context which should not affect the log path as a whole.  As far
    #   as the log path is concerned, the message was successfully
    #   dispatched to the destination.
    test_config = """

log {
    source(genmsg);
    destination(dest_dropall);
    destination(dest_after);
};
"""
    (file_true, file_false, file_after, file_fallback, file_dropall) = create_config(config, test_config)
    syslog_ng.start(config)

    assert "processed" not in file_dropall.get_stats()
    assert file_after.get_stats()["processed"] == 1
    assert "processed" not in file_fallback.get_stats()

    assert file_after.read_log() == "foobar\n"


def test_midpoint_inline_destination_that_drops_all_messages_does_not_cause_message_to_be_unmatched(config, syslog_ng):
    # Rationale:
    #
    #   midpoint destinations which drop messages completely do it in their
    #   own context which should not affect the log path as a whole.  As far
    #   as the log path is concerned, the message was successfully
    #   dispatched to the destination.
    test_config = """

log {
    source(genmsg);
    destination {
        channel { filter { false(); }; };
    };
    destination(dest_after);
};
"""
    (file_true, file_false, file_after, file_fallback, file_dropall) = create_config(config, test_config)
    syslog_ng.start(config)

    assert "processed" not in file_dropall.get_stats()
    assert file_after.get_stats()["processed"] == 1
    assert "processed" not in file_fallback.get_stats()

    assert file_after.read_log() == "foobar\n"


def test_filter_between_destinations_that_drops_all_messages_causes_the_message_to_be_unmatched(config, syslog_ng):
    # Rationale:
    #
    #   In this case the filter() is part of the log path and the path is
    #   not processed completely because the filter dropped the message.
    #   The anticipated use-case here is that the first destination serves
    #   debugging purposes only, and the delivery of the message to that
    #   destination does not mean that the log path was successfully
    #   finished.

    test_config = """

log {
    source(genmsg);
    destination(dest_true);
    filter { false(); };
    destination(dest_after);
};
"""
    (file_true, file_false, file_after, file_fallback, _) = create_config(config, test_config)
    syslog_ng.start(config)

    assert file_true.get_stats()["processed"] == 1
    assert "processed" not in file_after.get_stats()
    assert file_fallback.get_stats()["processed"] == 1

    assert file_true.read_log() == "foobar\n"
    assert file_fallback.read_log() == "foobar\n"


def test_junction_that_drops_messages_in_all_branches_causes_the_message_to_be_unmatched(config, syslog_ng):

    test_config = """

log {
    source(genmsg);
    destination(dest_true);
    junction {
        channel { filter { false(); }; };
        channel { filter { false(); }; };
    };
    destination(dest_after);
};
"""
    (file_true, file_false, file_after, file_fallback, _) = create_config(config, test_config)
    syslog_ng.start(config)

    assert file_true.get_stats()["processed"] == 1
    assert "processed" not in file_after.get_stats()
    assert file_fallback.get_stats()["processed"] == 1

    assert file_true.read_log() == "foobar\n"
    assert file_fallback.read_log() == "foobar\n"


def test_junction_that_drops_messages_in_all_branches_causes_the_message_to_be_unmatched_even_if_they_reference_destinations(config, syslog_ng):

    test_config = """

log {
    source(genmsg);
    junction {
        channel {
            destination(dest_true);
            filter { false(); };
            destination(dest_false);
        };
        channel { filter { false(); }; };
    };
    destination(dest_after);
};
"""
    (file_true, file_false, file_after, file_fallback, _) = create_config(config, test_config)
    syslog_ng.start(config)

    assert file_true.get_stats()["processed"] == 1
    assert "processed" not in file_false.get_stats()
    assert "processed" not in file_after.get_stats()
    assert file_fallback.get_stats()["processed"] == 1

    assert file_true.read_log() == "foobar\n"
    assert file_fallback.read_log() == "foobar\n"


def test_junction_that_contains_message_dropping_destination_continues_to_deliver_messages_on_the_same_path(config, syslog_ng):

    test_config = """

log {
    source(genmsg);
    junction {
        channel {
            destination(dest_dropall);
            destination(dest_true);
        };
        channel { filter { false(); }; destination(dest_false); };
    };
    destination(dest_after);
};
"""
    (file_true, file_false, file_after, file_fallback, file_dropall) = create_config(config, test_config)
    syslog_ng.start(config)

    assert "processed" not in file_dropall.get_stats()
    assert file_true.get_stats()["processed"] == 1
    assert "processed" not in file_false.get_stats()

    assert file_after.get_stats()["processed"] == 1
    assert "processed" not in file_fallback.get_stats()

    assert file_true.read_log() == "foobar\n"
    assert file_after.read_log() == "foobar\n"



#!/usr/bin/env python
#############################################################################
# Copyright (c) 2007-2015 Balabit
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 2 as published
# by the Free Software Foundation, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#
# As an additional exemption you are allowed to compile & link against the
# OpenSSL libraries as published by the OpenSSL project. See the file
# COPYING for details.
#
#############################################################################

import os, errno

#internal modules
from log import *
from messagegen import *
from control import *
from globals import *
import messagegen



def init_env():
    for pipe in ('log-pipe', 'log-padded-pipe'):
        try:
            os.unlink(pipe)
        except OSError:
            pass
        os.mkfifo(pipe)
    try:
        os.mkdir("wildcard")
    except OSError as e:
        if e.errno != errno.EEXIST:
            raise

def get_seed_input():
    raw_input =  """k/zFvqjGWdhStmhfOeTNtTs89P8soknF1J9kSQrz8hKdrjIutqTXMfIqCNUb7DXrMykMW+wKd1Pg
DwaUwxKmlaU1ItOek+jNUWVw9ZOSI1EmsXVgu+Hu7URgmeyY0A3WsDmMzR0Z2wTcRFSuINgBP8LC
8SG27gJZVOoVv09pfY9WyjvUYwg1jBdTfEM+qcDQKOACx4DH+SzO0bOOJMfMbR2iFaq18b/TCeQN
kRy9Lz2WvBsByQoXw/afxiu5xzn0MHoxTMCZCTjIyhGXzO/R2yj3eBVc5vxc5oxG3/EdjGnhmn/L
HEFVgc6EJ5OF1ye8hDIHFdZFehCAPbso3LL/3r8oJn+Axmc2sOrvhzMLpCV2KWdy8haiv3WZ9hZP
iJkC0pGBR+vhrdYE7qh2RzcdYoHRCvX5tpaOG+SE8+Az2WPHPOD7j4j+4KbhW+YbzYfUAsmwOQb4
R3vIyGO6Og4fg+AmKcycpv9Bo17zidbQe9zdZO6X4Df+ychK+yRSCMHLUyrEry7IFt60ivLAjc8b
Ladp70v7icWO+J9P/3pXIzKuQuaeT7J1q2xlCc/srV2pNV4+EhKJ8qSe+hG4LzpXWRGKo72h6BvL
Wgcp3S/wy1e6ov8XsVKjkWeQH8Nh3xOMWGYh6/yXSN44BLIBUdqk4I3TCKy1DawJQd1ivytEF0Xh
BstGzSoyeR92mN/K5/gy9wKFZffTbE7DmysKflRAN85ht7IVqxrTNXQ+UPWvlGZDAQ0NY45rQI3K
S1q0ahQivgGUZmEhuZkAUlYGWAqtCeDz3rZvrp5r2WJFGTZ+9yJZC5T2L+erDGBYxmcwxDzz8AvS
Ybp4aIBEtTK71cx9DtFTtSaQ6aW9rI5nc/owo0gv4Ddm5BYjMAd7kcc9TWnb1DZ9AJQAnSxgcuJM
acbYkFcqtMnafh/VnGekZ8yM0ZZqQPKBhysx+u2UBXib7Vvb6x6Y/xglNcqBWGFmzruKt6hx0pR2
x9IunzeIwdlcwIhLEKfIiy9ULwi7RTjVSeqgwucWoC0lw0BTotGeLDcFxTlQsE3T/UneLa8H6iSH
VW5iRZrvI0sdxt5Ud0TjNqXRGxuVczSuwpQwwxBn0ogr9DoRnp375PwGGh1/yqimW/+OStwP3cRR
yXEg6Zq1CvuYF/E6el4h9GylxkU7wEM2Ti9QJY4n3YsHyesalERqdd9xx5t7ADRodpMpZXoZGbrS
vccp3zMzS/aEZRuxky1/qjrAEh8OVA58e82jQqTdY8OQ/kKOu/gUgKBnHAvLkB/020p0CNbq6HjY
l625DLckaYmOPTh0ECFKzhaPF+/LNmzD36ToOAeuNjfbUjiUVGfntr2mc4E8mUFyo+TskrkSfw==
""".encode()

    import sys
    import base64
    if sys.version_info.major == 2:
      return base64.decodestring(raw_input)
    else:
      return base64.decodebytes(raw_input)


def seed_rnd():
    # Some platforms lack kernel supported random numbers, on those we have to explicitly seed the RNG.
    # using a fixed RND input does not matter as we are not protecting real data in this test program.
    try:
        import socket
        import ssl
        rnd = get_seed_input()
        ssl.RAND_add(rnd, 1024)
        if not ssl.RAND_status():
            raise "PRNG not seeded"
    except ImportError:
        return

def run_testcase(test_name, config, verbose, test_case):
    print_start(test_name)

    if not start_syslogng(config, verbose):
      sys.exit(1)

    print_user("Starting test case...")
    success = test_case()
    if not stop_syslogng():
      sys.exit(1)
    print_end(test_name, success)
    return success

# import test modules
import test_file_source
import test_filters
import test_input_drivers
import test_performance
import test_sql
import test_python
import test_map_value_pairs

tests = (test_input_drivers, test_sql, test_file_source, test_filters, test_performance, test_python, test_map_value_pairs)
failed_tests = []

init_env()
seed_rnd()


verbose = False
if len(sys.argv) > 1:
    verbose = True
try:
    for test_module in tests:
        if hasattr(test_module, "check_env") and not test_module.check_env():
            continue


        contents = dir(test_module)
        contents.sort()
        for obj in contents:
            if obj[:5] != 'test_':
                continue
            test_case = getattr(test_module, obj)

            if type(test_module.config) is str:
                test_case_name = test_module.__name__ + '.' + obj
                if not run_testcase(test_case_name, test_module.config, verbose, test_case):
                    failed_tests.append(test_case_name)

            elif type(test_module.config) is dict:
                for config_name in test_module.config:
                    testcase_name = "%s.%s[%s]" %(test_module.__name__, obj, config_name)
                    config = test_module.config[config_name]
                    if not run_testcase(testcase_name, config, verbose, test_case):
                        failed_tests.append(testcase_name)

finally:
    stop_syslogng()

if len(failed_tests)>0:
    print("List of failed test cases: %s" % ','.join(failed_tests))
    sys.exit(1)

sys.exit(0)



#!/usr/bin/env python
#############################################################################
# Copyright (c) 2020 One Identity
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 2 as published
# by the Free Software Foundation, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#
# As an additional exemption you are allowed to compile & link against the
# OpenSSL libraries as published by the OpenSSL project. See the file
# COPYING for details.
#
#############################################################################

TEMPLATE = r'"${PROXIED_SRCIP} ${PROXIED_DSTIP} ${PROXIED_SRCPORT} ${PROXIED_DSTPORT} ${PROXIED_IP_VERSION} ${MESSAGE}\n"'
INPUT_MESSAGES = "PROXY TCP4 1.1.1.1 2.2.2.2 3333 4444\r\n" \
                 "message 0"
EXPECTED_MESSAGE0 = "1.1.1.1 2.2.2.2 3333 4444 4 message 0\n"


def test_pp_acceptance(config, syslog_ng, loggen, port_allocator):
    network_source = config.create_network_source(ip="localhost", port=port_allocator(), transport='"proxied-tcp"', flags="no-parse")
    file_destination = config.create_file_destination(file_name="output.log", template=TEMPLATE)
    config.create_logpath(statements=[network_source, file_destination])

    syslog_ng.start(config)

    network_source.write_log(INPUT_MESSAGES)

    assert file_destination.read_log() == EXPECTED_MESSAGE0



#!/usr/bin/env python
#############################################################################
# Copyright (c) 2020 Balabit
# Copyright (c) 2020 Kokan
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 2 as published
# by the Free Software Foundation, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#
# As an additional exemption you are allowed to compile & link against the
# OpenSSL libraries as published by the OpenSSL project. See the file
# COPYING for details.
#
#############################################################################
import pytest

test_parameters_raw = [
    (r"""<134>1 2018-03-21 17:25:25 MDS-72 CheckPoint 13752 - [action:"Update"; flags:"150784"; ifdir:"inbound"; logid:"160571424"; loguid:"{0x5ab27965,0x0,0x5b20a8c0,0x7d5707b6}"; origin:"192.168.32.91"; originsicname:"CN=GW91,O=Domain2_Server..cuggd3"; sequencenum:"1"; time:"1521645925"; version:"5"; auth_method:"Machine Authentication (Active Directory)"; auth_status:"Successful Login"; authentication_trial:"this is a reauthentication for session 9a026bba"; client_name:"Active Directory Query"; client_version:"R80.10"; domain_name:"spec.mgmt"; endpoint_ip:"192.168.32.69"; identity_src:"AD Query"; identity_type:"machine"; product:"Identity Awareness"; snid:"9a026bba"; src:"192.168.32.69"; src_machine_group:"All Machines"; src_machine_name:"yonatanad";]""", "<CheckPoint><MDS-72><Mar 21 17:25:25>"),
    (r"""<134>1 2018-03-21T17:25:25 MDS-72 CheckPoint 13752 - [action:"Update"; flags:"150784"; ifdir:"inbound"; logid:"160571424"; loguid:"{0x5ab27965,0x0,0x5b20a8c0,0x7d5707b6}"; origin:"192.168.32.91"; originsicname:"CN=GW91,O=Domain2_Server..cuggd3"; sequencenum:"1"; time:"1521645925"; version:"5"; auth_method:"Machine Authentication (Active Directory)"; auth_status:"Successful Login"; authentication_trial:"this is a reauthentication for session 9a026bba"; client_name:"Active Directory Query"; client_version:"R80.10"; domain_name:"spec.mgmt"; endpoint_ip:"192.168.32.69"; identity_src:"AD Query"; identity_type:"machine"; product:"Identity Awareness"; snid:"9a026bba"; src:"192.168.32.69"; src_machine_group:"All Machines"; src_machine_name:"yonatanad";]""", "<CheckPoint><MDS-72><Mar 21 17:25:25>"),
    (r"""<134>1 2019-11-27T02:58:25Z ABDP-CPLOG01 CheckPoint 22103 - [action:"Accept"; flags:"18692"; ifdir:"inbound"; ifname:"bond1.734"; loguid:"{0x5ddde651,0x48,0xca96040a,0xc000001b}"; origin:"10.4.150.2"; time:"1574823505"; version:"1"; __policy_id_tag:"product=VPN-1 & FireWall-1[db_tag={12D7A082-42F0-B240-8103-29F2F6FF139C};mgmt=ABDP-MGT01;date=1573204576;policy_name=Cutover_Final_V3_20161031]"; dst:"124.156.190.9"; nat_addtnl_rulenum:"1"; nat_rulenum:"8"; origin_sic_name:"CN=ABDP-CPFW01,O=ABDP-CPMGT01..94r78q"; product:"VPN-1 & FireWall-1"; proto:"6"; rule:"8"; rule_name:"3G IP Pool Outgoing"; rule_uid:"{DF64DC15-BEDF-4246-8B71-A75E0991C5D9}"; s_port:"64442"; service:"80"; service_id:"http"; src:"10.7.79.7"; xlatedport:"0"; xlatedst:"0.0.0.0"; xlatesport:"49254"; xlatesrc:"202.1.50.67"; ]""", "<CheckPoint><ABDP-CPLOG01><Nov 27 02:58:25>"),
    (r"""<134>1 2020-01-02T14:04:50Z fwmgmt CheckPoint 16559 - [action:"Accept"; flags:"411908"; ifdir:"inbound"; ifname:"bond0.401"; logid:"0"; loguid:"{0x5e0df882,0x0,0x20010ac,0xc0000002}"; origin:"172.16.0.2"; originsicname:"CN=fw01,O=fwmgmt.ny-net.local.nahkt6"; sequencenum:"6"; time:"1577973890"; version:"5"; __policy_id_tag:"product=VPN-1 & FireWall-1[db_tag={E97DE696-700C-7F42-BD71-537C8A0883CB};mgmt=fwmgmt;date=1556607233;policy_name=Standard\]"; dst:"192.203.230.10"; inzone:"Internal"; layer_name:"Network"; layer_uuid:"0eeae4c4-aa05-40a9-a319-623823330c95"; match_id:"7"; parent_rule:"0"; rule_action:"Accept"; rule_name:"ALLOW"; rule_uid:"52ff639d-c772-4797-8b45-e151e295702e"; nat_addtnl_rulenum:"1"; nat_rulenum:"8"; outzone:"External"; product:"VPN-1 & FireWall-1"; proto:"17"; s_port:"55263"; service:"53"; service_id:"domain-udp"; src:"10.126.0.154"; xlatedport:"0"; xlatedst:"0.0.0.0"; xlatesport:"52993"; xlatesrc:"172.16.16.10"; ]""", "<CheckPoint><fwmgmt><Jan  2 14:04:50>"),
]


@pytest.mark.parametrize(
    "input_message, expected_value", test_parameters_raw,
    ids=list(map(str, range(len(test_parameters_raw)))),
)
def test_checkpoint_parser(config, syslog_ng, input_message, expected_value):
    config.add_include("scl.conf")

    generator_source = config.create_example_msg_generator_source(num=1, template=config.stringify(input_message))
    checkpoint_parser = config.create_checkpoint_parser()

    file_destination = config.create_file_destination(file_name="output.log", template=config.stringify('<${PROGRAM}><${HOST}><${S_DATE}>\n'))
    config.create_logpath(statements=[generator_source, checkpoint_parser, file_destination])

    syslog_ng.start(config)
    assert file_destination.read_log().strip() == expected_value



#!/usr/bin/env python
#############################################################################
# Copyright (c) 2015-2018 Balabit
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 2 as published
# by the Free Software Foundation, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#
# As an additional exemption you are allowed to compile & link against the
# OpenSSL libraries as published by the OpenSSL project. See the file
# COPYING for details.
#
#############################################################################
import pytest

input_log = "<38>Feb 11 21:27:22 testhost testprogram[9999]: test message\n"
expected_log = "Feb 11 21:27:22 testhost testprogram[9999]: test message\n"


@pytest.mark.parametrize(
    "input_log, expected_log, counter", [
        (input_log, expected_log, 1),
        (input_log, expected_log, 10),
    ], ids=["with_one_log", "with_ten_logs"],
)
def test_acceptance(config, syslog_ng, input_log, expected_log, counter):
    file_source = config.create_file_source(file_name="input.log")
    file_destination = config.create_file_destination(file_name="output.log")
    config.create_logpath(statements=[file_source, file_destination])
    config.update_global_options(keep_hostname="yes")

    file_source.write_log(input_log, counter)
    syslog_ng.start(config)
    assert file_destination.read_logs(counter) == [expected_log] * counter
